import arrow
import click
from flask.cli import with_appcontext

from opencve.commands import ensure_config, info, timed_operation
from opencve.commands.imports import cpe, cve, cwe
from opencve.extensions import db
from opencve.models.cve import Cve
from opencve.models.metas import Meta

CURRENT_YEAR = arrow.now().year
CVE_FIRST_YEAR = 2002


@click.command()
@click.option("--confirm", is_flag=True, help="Automatically confirm the prompt.")
@ensure_config
@with_appcontext
def import_data(confirm):
    """
    Perform initial imports (cve, cwe, cpe).
    """
    if Cve.query.first():
        info("Import already done.")
        return

    if not confirm:
        msg = (
            "This command will import initial data in your database. "
            "Do you want to continue ?".format(CVE_FIRST_YEAR, CURRENT_YEAR)
        )
        if not click.confirm(msg):
            info("Bye.")
            return

    # Import the CWE list
    cwe.run()

    # Import the CVE, then use the returned list of vendors
    # to merge them with the official CPE dictionnary list.
    vendors = cve.run()
    cpe.run(vendors)

    # Populate the metas table
    with timed_operation("Populating metas table..."):
        meta = Meta(name="nvd_last_sha256", value="default")
        db.session.add(meta)
        db.session.commit()



from flask import abort
from sqlalchemy.orm import joinedload

from opencve.controllers.base import BaseController
from opencve.models.alerts import Alert


class AlertController(BaseController):
    model = Alert
    order = [Alert.created_at.desc()]
    per_page_param = "ALERTS_PER_PAGE"
    schema = {
        "user_id": {"type": str},
        "report_id": {"type": str},
    }

    @classmethod
    def build_query(cls, args):
        query = cls.model.query.options(joinedload("cve")).filter_by(
            report_id=args.get("report_id")
        )
        return query, {}



from datetime import datetime

import pytest
from unittest.mock import patch, MagicMock
from flask_user import EmailError

from opencve.extensions import db
from opencve.models.alerts import Alert
from opencve.models.vendors import Vendor
from opencve.models.reports import Report
from opencve.tasks.alerts import handle_alerts
from opencve.tasks.reports import (
    get_top_alerts,
    get_sorted_alerts,
    get_users_with_alerts,
    get_vendors_products,
    handle_reports,
    reports_cleanup,
)


@pytest.mark.parametrize(
    "hour,count",
    [
        ("02:00:00", 1),
        ("10:59:00", 1),
        ("11:00:00", 2),
        ("11:15:00", 2),
        ("11:16:00", 1),
        ("20:00:00", 1),
    ],
)
def test_get_users_with_alerts(freezer, create_user, handle_events, hour, count):
    handle_events("modified_cves/CVE-2018-18074.json")

    # Create 2 users with different frequency notification
    user1 = create_user("user1")
    user1.vendors.append(Vendor.query.filter_by(name="canonical").first())
    user1.frequency_notifications = "always"
    db.session.commit()

    user2 = create_user("user2")
    user2.vendors.append(Vendor.query.filter_by(name="canonical").first())
    user2.frequency_notifications = "once"
    db.session.commit()

    handle_alerts()

    freezer.move_to(f"2021-01-01 {hour}")
    users = get_users_with_alerts()
    assert len(users) == count


def test_get_top_alerts(create_cve, create_user):
    user = create_user()
    db.session.add(Alert(cve=create_cve("CVE-2018-18074"), user=user, details={}))
    db.session.add(Alert(cve=create_cve("CVE-2020-9392"), user=user, details={}))
    db.session.add(Alert(cve=create_cve("CVE-2020-26116"), user=user, details={}))
    db.session.commit()

    # List of alerts is reduced and ordered by CVSS3 desc
    top_alerts = get_top_alerts(user, 1)
    assert [a.cve.cvss3 for a in top_alerts] == [9.8]

    top_alerts = get_top_alerts(user, 3)
    assert sorted([a.cve.cvss3 for a in top_alerts]) == sorted([9.8, 7.3, 7.2])

    top_alerts = get_top_alerts(user, 10)
    assert sorted([a.cve.cvss3 for a in top_alerts]) == sorted([9.8, 7.3, 7.2])


def test_get_sorted_alerts(create_cve, create_user):
    user = create_user()

    # Create an alert with the 'foo' vendor
    alert_26116 = Alert(
        cve=create_cve("CVE-2020-26116"),
        user=user,
        details={"vendors": ["foo"], "products": []},
    )
    db.session.add(alert_26116)
    db.session.commit()

    alerts = Alert.query.all()
    sorted_alerts = get_sorted_alerts(alerts)
    assert "foo" in sorted_alerts
    assert sorted_alerts["foo"]["name"] == "Foo"
    assert sorted_alerts["foo"]["max"] == 7.2
    assert [a.id for a in sorted_alerts["foo"]["alerts"]] == [alert_26116.id]

    # Add another alert for the same 'foo' vendor but with a higher score
    alert_28074 = Alert(
        cve=create_cve("CVE-2018-18074"),
        user=user,
        details={"vendors": ["foo"], "products": []},
    )
    db.session.add(alert_28074)
    db.session.commit()

    alerts = Alert.query.all()
    sorted_alerts = get_sorted_alerts(alerts)
    assert "foo" in sorted_alerts
    assert sorted_alerts["foo"]["name"] == "Foo"
    assert sorted_alerts["foo"]["max"] == 9.8
    assert sorted([a.id for a in sorted_alerts["foo"]["alerts"]]) == sorted(
        [alert_26116.id, alert_28074.id]
    )

    # Finally create an alert with the 'bar' product
    alert_9392 = Alert(
        cve=create_cve("CVE-2020-9392"),
        user=user,
        details={"vendors": [], "products": ["bar"]},
    )
    db.session.add(alert_9392)
    db.session.commit()

    alerts = Alert.query.all()
    sorted_alerts = get_sorted_alerts(alerts)
    assert "bar" in sorted_alerts
    assert sorted_alerts["bar"]["name"] == "Bar"
    assert sorted_alerts["bar"]["max"] == 7.3
    assert [a.id for a in sorted_alerts["bar"]["alerts"]] == [alert_9392.id]


def test_get_vendors_products(create_cve, create_user):
    user = create_user()
    db.session.add(
        Alert(
            cve=create_cve("CVE-2020-26116"),
            user=user,
            details={"vendors": ["foo"], "products": []},
        )
    )
    db.session.add(
        Alert(
            cve=create_cve("CVE-2018-18074"),
            user=user,
            details={"vendors": ["foo"], "products": []},
        )
    )
    db.session.add(
        Alert(
            cve=create_cve("CVE-2020-9392"),
            user=user,
            details={"vendors": [], "products": ["bar"]},
        )
    )
    db.session.commit()

    vendors_products = get_vendors_products(Alert.query.all())
    assert sorted(vendors_products) == sorted(["bar", "foo"])


def test_server_name_exceptions(app):
    old = app.config["SERVER_NAME"]
    app.config["SERVER_NAME"] = None

    with pytest.raises(ValueError):
        handle_reports()

    app.config["SERVER_NAME"] = old


@patch("opencve.tasks.reports.user_manager.email_manager.send_user_report")
def test_report_without_notification(mock_send, create_user, handle_events):
    handle_events("modified_cves/CVE-2018-18074.json")

    user = create_user()
    user.vendors.append(Vendor.query.filter_by(name="canonical").first())
    user.enable_notifications = False
    db.session.commit()

    handle_alerts()
    handle_reports()

    reports = Report.query.all()
    assert len(reports) == 1
    assert reports[0].user.id == user.id
    assert reports[0].details == ["canonical"]
    assert reports[0].alerts == Alert.query.filter_by(user_id=user.id).all()

    assert not mock_send.called
    assert Alert.query.filter_by(notify=False).count() == 0


@patch("opencve.tasks.reports.user_manager.email_manager.send_user_report")
def test_report_with_notification(mock_send, create_user, handle_events):
    handle_events("modified_cves/CVE-2018-18074.json")

    user = create_user()
    user.vendors.append(Vendor.query.filter_by(name="canonical").first())
    db.session.commit()

    handle_alerts()
    handle_reports()

    reports = Report.query.all()
    assert len(reports) == 1
    assert reports[0].user.id == user.id
    assert reports[0].details == ["canonical"]
    assert reports[0].alerts == Alert.query.filter_by(user_id=user.id).all()

    assert mock_send.called
    mock_send.assert_called_with(
        user,
        **{
            "subject": "1 alert on Canonical",
            "total_alerts": 1,
            "alerts_sorted": get_sorted_alerts(Alert.query.all()),
            "report_public_link": Report.query.first().public_link,
        },
    )
    assert Alert.query.filter_by(notify=False).count() == 0


@patch("opencve.tasks.reports.user_manager.email_manager.send_user_report")
def test_report_bad_smtp_config(mock_send, create_user, handle_events):
    mock_send.side_effect = EmailError("error")

    handle_events("modified_cves/CVE-2018-18074.json")

    user = create_user()
    user.vendors.append(Vendor.query.filter_by(name="canonical").first())
    db.session.commit()

    handle_alerts()
    handle_reports()

    reports = Report.query.all()
    assert len(reports) == 1
    assert reports[0].user.id == user.id
    assert reports[0].details == ["canonical"]
    assert reports[0].alerts == Alert.query.filter_by(user_id=user.id).all()
    assert len(reports[0].alerts) == 1
    assert Alert.query.filter_by(notify=False).count() == 0


@pytest.mark.freeze_time("2022-09-19")
def test_reports_cleanup(create_user, create_cve):
    user = create_user()

    alert_2018_18074 = Alert(cve=create_cve("CVE-2018-18074"), user=user, details={})
    alert_2020_9392 = Alert(cve=create_cve("CVE-2020-9392"), user=user, details={})
    alert_2020_26116 = Alert(cve=create_cve("CVE-2020-26116"), user=user, details={})

    # ~/tests/opencve.cfg::reports_cleanup_days = 7
    old_report = Report(
        created_at=datetime(2022, 9, 10),
        user=user,
        alerts=[alert_2018_18074, alert_2020_9392],
    )
    fresh_report = Report(
        created_at=datetime(2022, 9, 15), user=user, alerts=[alert_2020_26116]
    )

    db.session.add(alert_2018_18074)
    db.session.add(alert_2020_9392)
    db.session.add(alert_2020_26116)
    db.session.add(old_report)
    db.session.add(fresh_report)
    db.session.commit()

    assert Report.query.count() == 2
    assert Alert.query.count() == 3

    reports_cleanup()

    reports = Report.query.all()
    assert len(reports) == 1
    assert reports[0].id == fresh_report.id

    alerts = Alert.query.all()
    assert len(alerts) == 1
    assert alerts[0].id == alert_2020_26116.id
    assert alerts[0].report_id == fresh_report.id



import pytest
from werkzeug.exceptions import NotFound

from unittest.mock import patch

from opencve.extensions import db
from opencve.models.alerts import Alert
from opencve.models.reports import Report
from opencve.models.vendors import Vendor
from opencve.controllers.reports import ReportController
from opencve.tasks.alerts import handle_alerts
from opencve.tasks.reports import handle_reports


def test_metas(app):
    with app.test_request_context():
        _, metas, _ = ReportController.list()
    assert metas == {}


@patch("opencve.tasks.reports.user_manager.email_manager.send_user_report")
def test_list(mock_send, app, handle_events, create_user):
    handle_events("modified_cves/CVE-2018-18074.json")

    user = create_user()
    user.vendors.append(Vendor.query.filter_by(name="canonical").first())
    db.session.commit()

    handle_alerts()
    handle_reports()

    with app.test_request_context():
        reports = ReportController.list_items({"user_id": user.id})
    assert len(reports) == 1
    assert reports[0].user.id == user.id
    assert reports[0].details == ["canonical"]
    assert reports[0].alerts == Alert.query.filter_by(user_id=user.id).all()


@patch("opencve.tasks.reports.user_manager.email_manager.send_user_report")
def test_list_paginated(mock_send, app, handle_events, create_user):
    old = app.config["REPORTS_PER_PAGE"]
    app.config["REPORTS_PER_PAGE"] = 2

    user = create_user()
    db.session.commit()

    handle_events("modified_cves/CVE-2018-18074.json")
    user.vendors.append(Vendor.query.filter_by(name="canonical").first())
    db.session.commit()
    handle_alerts()
    handle_reports()

    handle_events("modified_cves/CVE-2019-17052.json")
    user.vendors.append(Vendor.query.filter_by(name="linux").first())
    db.session.commit()
    handle_alerts()
    handle_reports()

    handle_events("modified_cves/CVE-2020-26116.json")
    user.vendors.append(Vendor.query.filter_by(name="python").first())
    db.session.commit()
    handle_alerts()
    handle_reports()

    with app.test_request_context():
        reports = ReportController.list_items({"user_id": user.id})
        assert len(reports) == 2
        assert reports[0].details == ["python"]
        assert reports[1].details == ["linux"]

        reports = ReportController.list_items({"user_id": user.id, "page": 2})
        assert len(reports) == 1
        assert reports[0].details == ["canonical"]

    app.config["REPORTS_PER_PAGE"] = old


@patch("opencve.tasks.reports.user_manager.email_manager.send_user_report")
def test_get(mock_send, app, handle_events, create_user):
    handle_events("modified_cves/CVE-2018-18074.json")

    user = create_user()
    user.vendors.append(Vendor.query.filter_by(name="canonical").first())
    db.session.commit()

    handle_alerts()
    handle_reports()

    report_1 = Report.query.first()
    assert report_1.seen == False

    with app.test_request_context():
        report_2 = ReportController.get({"public_link": report_1.public_link})
    assert report_1.id == report_2.id
    assert report_2.seen == True



from flask import request
from flask_restful import fields, marshal_with

from opencve.api.base import BaseResource
from opencve.api.cves import cves_fields
from opencve.controllers.cves import CveController
from opencve.controllers.cwes import CweController


cwes_fields = {
    "id": fields.String(attribute="cwe_id"),
    "name": fields.String(attribute="name"),
    "description": fields.String(attribute="description"),
}


class CweListResource(BaseResource):
    @marshal_with(cwes_fields)
    def get(self):
        return CweController.list_items(request.args)


class CweResource(BaseResource):
    @marshal_with(cwes_fields)
    def get(self, id):
        return CweController.get({"cwe_id": id})


class CweCveResource(BaseResource):
    @marshal_with(cves_fields)
    def get(self, id):
        CweController.get({"cwe_id": id})
        return CveController.list_items({**request.args, "cwe": id})



"""Synchronize null CVSS for existing CVEs

Revision ID: f81abceece3d
Revises: 41ad8e9163d4
Create Date: 2021-01-16 18:34:17.488053

"""

# revision identifiers, used by Alembic.
revision = "f81abceece3d"
down_revision = "41ad8e9163d4"

import datetime

from alembic import op
import sqlalchemy as sa
from sqlalchemy.ext.declarative import declarative_base

from opencve.models.cve import Cve
from opencve.models.events import Event


def upgrade():
    """
    This migration is only useful for users who are upgrading
    from a previous version of OpenCVE.
    It fixes a bug when CVSS fields were not well filled.
    """
    connection = op.get_bind()
    sessionmaker = sa.orm.sessionmaker()
    session = sessionmaker(bind=connection)

    # Fetch the CVEs linked to CVSS events. Only the
    # events with an empty 'new' field are kept
    cves = [
        e.cve
        for e in session.query(Event).filter(Event.type == "cvss").all()
        if not e.details["new"]
    ]
    if not cves:
        return

    # Clean their cvss2 and cvss3 fields
    for cve in cves:
        cve.cvss2 = None
        cve.cvss3 = None

        # Small tip to keep the same updated_at attribute
        cve.updated_at = cve.updated_at + datetime.timedelta(seconds=1)
        session.commit()
        cve.updated_at = cve.updated_at + datetime.timedelta(seconds=-1)
        session.commit()


def downgrade():
    pass



from unittest.mock import patch

from opencve.commands.upgrade_db import upgrade_db
from opencve.extensions import db
from opencve.models.cve import Cve


def test_upgrade_db_command(app):
    runner = app.test_cli_runner()
    result = runner.invoke(upgrade_db, [])

    # Migrations are executed
    assert (
        "INFO  [alembic.runtime.migration] Context impl PostgresqlImpl."
        in result.output
    )
    assert (
        "INFO  [alembic.runtime.migration] Will assume transactional DDL."
        in result.output
    )
    assert "INFO  [alembic.runtime.migration] Running upgrade" in result.output



import string

from flask import abort

from opencve.controllers.base import BaseController
from opencve.models.cve import Cve
from opencve.models.cwe import Cwe


class CweController(BaseController):
    model = Cwe
    order = [Cwe.cwe_id.desc()]
    per_page_param = "CWES_PER_PAGE"
    schema = {
        "search": {"type": str},
    }

    @classmethod
    def build_query(cls, args):
        query = cls.model.query

        # Filter the list of CWE
        if args.get("search"):
            search = args.get("search").strip().lower()

            # By ID or by string
            search = search[4:] if search.startswith("cwe-") else search
            try:
                search = int(search)
                query = query.filter_by(cwe_id=f"CWE-{search}")
            except ValueError:
                query = query.filter(Cwe.name.ilike("%{}%".format(search)))

        return query, {}



import os

import click
from flask.cli import with_appcontext

from opencve.commands import ensure_config


@click.group()
def celery():
    """Run Celery commands."""


@celery.command(context_settings=dict(ignore_unknown_options=True))
@click.argument("args", nargs=-1, type=click.UNPROCESSED)
@ensure_config
@with_appcontext
def worker(args):
    """Run a Celery worker."""
    args = "celery worker -A opencve.app:cel".split() + list(args)
    os.execvp(args[0], args)


@celery.command(context_settings=dict(ignore_unknown_options=True))
@click.argument("args", nargs=-1, type=click.UNPROCESSED)
@ensure_config
@with_appcontext
def beat(args):
    """Start the Celery beat."""
    args = "celery beat -A opencve.app:cel".split() + list(args)
    os.execvp(args[0], args)



class BaseCheck(object):
    def __init__(self, cve_obj, cve_json):
        self.cve_obj = cve_obj
        self.cve_json = cve_json

    def get_name(self):
        return self.__class__.__name__

    def execute(self):
        raise NotImplementedError



# Used in the Events and Alerts tables
EVENT_TYPES = [
    ("new_cve", "New CVE"),
    ("first_time", "Vendors/Products appeared for the first time"),
    ("references", "References changed"),
    ("cpes", "CPEs changed"),
    ("cvss", "CVSS changed"),
    ("summary", "Summary changed"),
    ("cwes", "CWEs changed"),
]

# Used in the users model
FREQUENCIES_TYPES = [
    ("once", "Once a day"),
    ("always", "As soon as a change is detected"),
]

CVSS_SCORES = [
    (0, 0),
    (1, 1),
    (2, 2),
    (3, 3),
    (4, 4),
    (5, 5),
    (6, 6),
    (7, 7),
    (8, 8),
    (9, 9),
]

PRODUCT_SEPARATOR = "$PRODUCT$"

# This is the message sent by Flask-User in the `flash` function
EMAIL_CONFIRMATION_MESSAGE = (
    "Your email address has not yet been confirmed. Check your email Inbox "
    'and Spam folders for the confirmation email or <a href="/account/resend-'
    'email-confirmation">Re-send confirmation email</a>.'
)



from pathlib import Path

from flask import send_from_directory
from flask import current_app as app

from opencve.controllers.main import main


@main.route("/favicon.ico")
def favicon():
    return send_from_directory(
        str(Path(app.root_path).resolve() / "static/img"),
        "favicon.ico",
        mimetype="image/vnd.microsoft.icon",
    )



from unittest.mock import patch

from opencve.commands.webserver import webserver


@patch("opencve.commands.webserver.os.execvp")
def test_webserver_command(mock_execvp, app):
    mock_execvp.return_value = True
    runner = app.test_cli_runner()

    # Without arguments
    runner.invoke(webserver, [])
    mock_execvp.assert_called_with("gunicorn", ["gunicorn", "opencve.app:app"])

    # With custom arguments
    runner.invoke(webserver, ["-b", "0.0.0.0:80"])
    mock_execvp.assert_called_with(
        "gunicorn", ["gunicorn", "-b", "0.0.0.0:80", "opencve.app:app"]
    )



import os

import click

from opencve import __version__
from opencve.commands.celery import celery
from opencve.commands.create_user import create_user
from opencve.commands.init import init
from opencve.commands.upgrade_db import upgrade_db
from opencve.commands.imports import import_data
from opencve.commands.webserver import webserver


@click.group()
@click.version_option(version=__version__)
def cli():
    """CVE Alerting Platform"""
    os.environ["FLASK_APP"] = "opencve.app:app"


cli.add_command(celery)
cli.add_command(create_user)
cli.add_command(import_data)
cli.add_command(init)
cli.add_command(upgrade_db)
cli.add_command(webserver)



from collections import OrderedDict

from flask import request, render_template
from flask_login import login_required
from flask_user import current_user

from opencve.context import _humanize_filter
from opencve.controllers.main import main
from opencve.controllers.alerts import AlertController
from opencve.controllers.reports import ReportController


@main.route("/reports")
@login_required
def reports():
    reports, _, pagination = ReportController.list(
        {**request.args, "user_id": current_user.id}
    )
    return render_template("reports.html", reports=reports, pagination=pagination)


@main.route("/reports/<link>")
def report(link):
    report = ReportController.get({"public_link": link})
    alerts = AlertController.list_items({"report_id": report.id})

    # List of vendors/products per alert
    alerts_sorted = {}
    for alert in alerts:

        for vendor in alert.details["vendors"]:
            if vendor not in alerts_sorted:
                alerts_sorted[vendor] = {
                    "name": _humanize_filter(vendor),
                    "alerts": [],
                    "max": 0,
                }
            alerts_sorted[vendor]["alerts"].append(alert)
        for product in alert.details["products"]:
            if product not in alerts_sorted:
                alerts_sorted[product] = {
                    "name": _humanize_filter(product),
                    "alerts": [],
                    "max": 0,
                }
            alerts_sorted[product]["alerts"].append(alert)

    # For each vendor, we take the max score
    for k, als in alerts_sorted.items():

        # Get the max score
        cvss = [al.cve.cvss3 for al in als["alerts"] if al.cve.cvss3]
        if cvss:
            alerts_sorted[k]["max"] = max(cvss)

    alerts_sorted = OrderedDict(
        sorted(alerts_sorted.items(), key=lambda i: i[1]["max"], reverse=True)
    )

    # Some stats
    total_alerts = len(alerts)
    maximum_score = max([v["max"] for k, v in alerts_sorted.items()])

    return render_template(
        "report.html",
        alerts_sorted=alerts_sorted,
        total_alerts=total_alerts,
        total_vendors_products=len(alerts_sorted.keys()),
        maximum_score=maximum_score,
        report=report,
    )



def test_list_cves_authentication(client, create_user):
    create_user("opencve")
    response = client.get("/api/cve")
    assert response.status_code == 401
    response = client.login("john").get("/api/cve")
    assert response.status_code == 401
    response = client.login("opencve").get("/api/cve")
    assert response.status_code == 200


def test_list_cves(client, create_user, create_cve):
    create_user("opencve")
    response = client.login("opencve").get("/api/cve")
    assert response.status_code == 200
    assert response.json == []

    create_cve("CVE-2018-18074")
    response = client.login("opencve").get("/api/cve")
    assert response.status_code == 200
    assert len(response.json) == 1
    assert response.json[0] == {
        "created_at": "2018-10-09T17:29:00Z",
        "id": "CVE-2018-18074",
        "summary": "The Requests package before 2.20.0 for Python sends an HTTP Authorization header to an http URI upon receiving a same-hostname https-to-http redirect, which makes it easier for remote attackers to discover credentials by sniffing the network.",
        "updated_at": "2019-10-03T00:03:00Z",
    }


def test_get_cve_not_found(client, create_user):
    create_user("opencve")
    response = client.login("opencve").get("/api/cve/404")
    assert response.status_code == 404
    assert response.json == {"message": "Not found."}


def test_get_cve(client, create_user, create_cve, open_file):
    create_user("opencve")
    create_cve("CVE-2018-18074")

    response = client.login("opencve").get("/api/cve/CVE-2018-18074")
    assert response.status_code == 200

    raw_nvd_data = response.json.pop("raw_nvd_data")
    assert response.json == {
        "created_at": "2018-10-09T17:29:00Z",
        "id": "CVE-2018-18074",
        "summary": "The Requests package before 2.20.0 for Python sends an HTTP Authorization header to an http URI upon receiving a same-hostname https-to-http redirect, which makes it easier for remote attackers to discover credentials by sniffing the network.",
        "updated_at": "2019-10-03T00:03:00Z",
        "cvss": {"v2": 5.0, "v3": 9.8},
        "cwes": ["CWE-522"],
        "vendors": {"canonical": ["ubuntu_linux"], "python-requests": ["requests"]},
    }

    data = open_file("cves/CVE-2018-18074.json")
    assert data == raw_nvd_data



from nested_lookup import nested_lookup
from difflib import HtmlDiff

from opencve.constants import PRODUCT_SEPARATOR
from opencve.models.cwe import Cwe


def convert_cpes(conf):
    """
    This function takes an object, extracts its CPE uris and transforms them into
    a dictionnary representing the vendors with their associated products.
    """
    uris = nested_lookup("cpe23Uri", conf) if not isinstance(conf, list) else conf

    # Create a list of tuple (vendor, product)
    cpes_t = list(set([tuple(uri.split(":")[3:5]) for uri in uris]))

    # Transform it into nested dictionnary
    cpes = {}
    for vendor, product in cpes_t:
        if vendor not in cpes:
            cpes[vendor] = []
        cpes[vendor].append(product)

    return cpes


def flatten_vendors(vendors):
    """
    Takes a list of nested vendors and products and flat them.
    """
    data = []
    for vendor, products in vendors.items():
        data.append(vendor)
        for product in products:
            data.append(f"{vendor}{PRODUCT_SEPARATOR}{product}")
    return data


def get_cwes(problems):
    """
    Takes a list of problems and return the CWEs ID.
    """
    return list(set([p["value"] for p in problems]))


def get_cwes_details(problems):
    """
    Takes a list of problems and return the CWEs along
    with the name of the vulnerability.
    """
    cwes = {}
    for cwe_id in get_cwes(problems):
        cwes[cwe_id] = None
        cwe = Cwe.query.filter_by(cwe_id=cwe_id).first()
        if cwe:
            cwes[cwe_id] = cwe.name
    return cwes


class CustomHtmlHTML(HtmlDiff):
    def __init__(self, *args, **kwargs):
        self._table_template = """
        <table class="table table-diff table-condensed">
            <thead>
                <tr>
                    <th colspan="2">Old JSON</th>
                    <th colspan="2">New JSON</th>
                </tr>
            </thead>
            <tbody>%(data_rows)s</tbody>
        </table>"""
        super().__init__(*args, **kwargs)

    def _format_line(self, side, flag, linenum, text):
        text = text.replace("&", "&amp;").replace(">", "&gt;").replace("<", "&lt;")
        text = text.replace(" ", "&nbsp;").rstrip()
        return '<td class="diff_header">%s</td><td class="break">%s</td>' % (
            linenum,
            text,
        )



from flask import current_app as app
from flask import flash, redirect, render_template, request, url_for
from flask_login import current_user, login_required

from opencve.controllers.main import main
from opencve.controllers.tags import UserTagController
from opencve.models.cve import Cve
from opencve.models.tags import CveTag, UserTag
from opencve.extensions import db
from opencve.forms import (
    ChangeEmailForm,
    ChangePasswordForm,
    FiltersNotificationForm,
    MailNotificationsForm,
    TagForm,
)


@main.route("/account/subscriptions", methods=["GET"])
@login_required
def subscriptions():
    return render_template("profiles/subscriptions.html")


@main.route("/account/notifications", methods=["GET", "POST"])
@login_required
def notifications():
    mail_notifications_form = MailNotificationsForm(
        obj=current_user,
        enable="yes" if current_user.enable_notifications else "no",
        frequency=current_user.frequency_notifications.code,
    )

    filters = current_user.filters_notifications or {"event_types": [], "cvss": 0}
    filters_notifications_form = FiltersNotificationForm(
        obj=current_user,
        new_cve=True if "new_cve" in filters["event_types"] else False,
        first_time=True if "first_time" in filters["event_types"] else False,
        references=True if "references" in filters["event_types"] else False,
        cvss=True if "cvss" in filters["event_types"] else False,
        cpes=True if "cpes" in filters["event_types"] else False,
        summary=True if "summary" in filters["event_types"] else False,
        cwes=True if "cwes" in filters["event_types"] else False,
        cvss_score=filters["cvss"],
    )

    if request.method == "POST":
        form_name = request.form["form-name"]

        if (
            form_name == "mail_notifications_form"
            and mail_notifications_form.validate()
        ):
            current_user.enable_notifications = (
                True if mail_notifications_form.enable.data == "yes" else False
            )
            current_user.frequency_notifications = (
                mail_notifications_form.frequency.data
            )
            db.session.commit()

            flash(
                "Your notifications setting has been changed successfully.", "success"
            )
            return redirect(url_for("main.notifications"))

        if (
            form_name == "filters_notifications_form"
            and filters_notifications_form.validate()
        ):
            filters = {
                "event_types": [],
                "cvss": filters_notifications_form.cvss_score.data,
            }

            for typ in [
                "new_cve",
                "first_time",
                "references",
                "cvss",
                "cpes",
                "cwes",
                "summary",
            ]:
                if getattr(filters_notifications_form, typ).data:
                    filters["event_types"].append(typ)

            current_user.filters_notifications = filters
            db.session.commit()

            flash(
                "Your notifications setting has been changed successfully.", "success"
            )
            return redirect(url_for("main.notifications"))

    return render_template(
        "profiles/notifications.html",
        mail_notifications_form=mail_notifications_form,
        filters_notifications_form=filters_notifications_form,
    )


@main.route("/account/tags", methods=["GET", "POST"])
@login_required
def tags():
    tags, _, pagination = UserTagController.list(
        {**request.args, "user_id": current_user.id}
    )
    tag_form = TagForm()

    # Form has been submitted
    if request.method == "POST" and tag_form.validate():

        # Check if the tag doesn't already exist
        if UserTag.query.filter_by(
            user_id=current_user.id, name=tag_form.name.data
        ).first():
            flash("This tag already exists.", "error")

        # Create the new tag
        else:
            tag = UserTag(
                user=current_user,
                name=tag_form.name.data,
                description=tag_form.description.data,
                color=tag_form.color.data,
            )
            db.session.add(tag)
            db.session.commit()

            flash(f"The tag {tag.name} has been successfully added.", "success")
            return redirect(
                url_for("main.edit_tag", tag=tag.name, page=request.args.get("page"))
            )

    return render_template(
        "profiles/tags.html",
        tags=tags,
        form=tag_form,
        pagination=pagination,
        mode="create",
    )


@main.route("/account/tags/<string:tag>", methods=["GET", "POST"])
@login_required
def edit_tag(tag):
    tag = UserTagController.get({"user_id": current_user.id, "name": tag})
    if not tag:
        return redirect(url_for("main.tags"))

    tag_form = TagForm(obj=tag, color=tag.color)

    if request.method == "POST" and tag_form.validate():

        # Prohibit name change
        if tag_form.name.data != tag.name:
            return redirect(url_for("main.tags"))

        # Update the tag
        tag_form.populate_obj(tag)
        tag.color = tag_form.color.data
        db.session.commit()

        flash(f"The tag {tag.name} has been successfully updated.", "success")
        return redirect(
            url_for("main.edit_tag", tag=tag.name, page=request.args.get("page"))
        )

    tags, _, pagination = UserTagController.list(
        {**request.args, "user_id": current_user.id}
    )

    return render_template(
        "profiles/tags.html",
        tags=tags,
        form=tag_form,
        pagination=pagination,
        mode="update",
    )


@main.route("/account/tags/<string:tag>/delete", methods=["GET", "POST"])
@login_required
def delete_tag(tag):
    tag = UserTagController.get({"user_id": current_user.id, "name": tag})
    if not tag:
        return redirect(url_for("main.tags"))

    count = (
        db.session.query(Cve.id)
        .join(CveTag)
        .filter(CveTag.user_id == current_user.id)
        .filter(CveTag.tags.contains([tag.name]))
        .count()
    )

    if count > 0:
        flash(
            f"The tag {tag.name} is still associated to {count} CVE(s), detach them before removing the tag.",
            "error",
        )
        return redirect(url_for("main.tags"))

    # Confirmation page
    if request.method == "GET":
        return render_template("profiles/delete_tag.html", tag=tag, count=count)

    # Delete the tag
    else:
        db.session.delete(tag)
        db.session.commit()
        flash(f"The tag {tag.name} has been deleted.", "success")
        return redirect(url_for("main.tags"))



import os

import click
from flask.cli import with_appcontext

from opencve.commands import ensure_config


@click.command(context_settings=dict(ignore_unknown_options=True))
@click.argument("args", nargs=-1, type=click.UNPROCESSED)
@ensure_config
@with_appcontext
def webserver(args):
    """Run the webserver."""
    args = ["gunicorn"] + list(args)
    args.append("opencve.app:app")
    os.execvp(args[0], args)



import warnings

from opencve.checks import BaseCheck
from opencve.commands.utils import CveUtil

# Disable useless warnings in deepdiff
with warnings.catch_warnings():
    from deepdiff import DeepDiff


class References(BaseCheck):
    def execute(self):
        payload = {"changed": [], "added": [], "removed": []}

        # List the old and new references then make the diff between them
        old_refs = {
            ref["url"]: ref
            for ref in self.cve_obj.json["cve"]["references"]["reference_data"]
        }
        new_refs = {
            ref["url"]: ref
            for ref in self.cve_json["cve"]["references"]["reference_data"]
        }
        diff = DeepDiff(old_refs, new_refs)

        # New references
        payload["added"] = [
            new_refs[r[6:-2]] for r in diff.get("dictionary_item_added", [])
        ]

        # Removed references
        payload["removed"] = [
            old_refs[r[6:-2]] for r in diff.get("dictionary_item_removed", [])
        ]

        # Changed references (the following code parses the DeepDiff result
        # and extracts the urls of modified references)
        modified_urls = list(
            set(
                [
                    r.split("'][")[0].split("['")[1]
                    for r in list(diff.get("values_changed", {}).keys())
                    + list(diff.get("iterable_item_added", {}).keys())
                ]
            )
        )
        payload["changed"] = [
            {"old": old_refs[url], "new": new_refs[url]} for url in modified_urls
        ]

        # Create the event with the references changes
        if payload["changed"] or payload["added"] or payload["removed"]:
            event = CveUtil.create_event(
                self.cve_obj, self.cve_json, "references", payload
            )
            return event

        return None



from flask import abort

from opencve.controllers.base import BaseController
from opencve.models.vendors import Vendor


class VendorController(BaseController):
    model = Vendor
    order = [Vendor.name.asc()]
    per_page_param = "VENDORS_PER_PAGE"
    schema = {
        "search": {"type": str},
    }

    @classmethod
    def build_query(cls, args):
        query = cls.model.query

        # Search by term
        if args.get("search"):
            search = (
                args.get("search")
                .lower()
                .replace("%", "")
                .replace("_", "")
                .replace(" ", "_")
            )
            query = query.filter(cls.model.name.like("%{}%".format(search)))

        return query, {}



import datetime
import json
import os
from base64 import b64encode
from pathlib import Path
from unittest.mock import patch, Mock

tests_directory = os.path.dirname(os.path.realpath(__file__))
os.environ["OPENCVE_CONFIG"] = str(Path(tests_directory) / "opencve.cfg")
os.environ["OPENCVE_WELCOME_FILES"] = str(
    Path(tests_directory).parent / "opencve/templates/_welcome"
)

import pytest
from bs4 import BeautifulSoup
from flask import url_for
from flask.testing import FlaskClient
from werkzeug.datastructures import Headers

from opencve import create_app
from opencve.commands.utils import CveUtil
from opencve.extensions import db
from opencve.models.cve import Cve
from opencve.models.cwe import Cwe
from opencve.models.tags import UserTag
from opencve.models.users import User
from opencve.models.vendors import Vendor
from opencve.models.products import Product
from opencve.tasks.events import handle_events as handle_events_task


class CustomClient(FlaskClient):
    def __init__(self, *args, **kwargs):
        self._auth = None
        super(CustomClient, self).__init__(*args, **kwargs)

    def login(self, username):
        credentials = bytes(f"{username}:password", encoding="utf8")
        self._auth = b64encode(credentials).decode("utf-8")
        return self

    def open(self, *args, **kwargs):
        headers = kwargs.pop("headers", Headers())

        if self._auth:
            headers.extend({"Authorization": f"Basic {self._auth}"})
            kwargs["headers"] = headers
        return super().open(*args, **kwargs)


@pytest.fixture(scope="module", autouse=True)
def app():
    _app = create_app("test")
    ctx = _app.app_context()
    ctx.push()

    # create the tables
    db.create_all()

    yield _app

    # drop the tables
    db.session.close()
    db.drop_all()


@pytest.fixture(scope="function", autouse=True)
def truncate_db_tables(app):
    # Truncate all tables
    meta = db.metadata
    for table in reversed(meta.sorted_tables):
        db.session.execute(table.delete())
    db.session.commit()


@pytest.fixture(scope="function")
def client(app):
    app.test_client_class = CustomClient
    _client = app.test_client()
    yield _client


@pytest.fixture(scope="function")
def open_file():
    def _open_file(name):
        with open(Path(__file__).parent.resolve() / "data" / name) as f:
            return json.load(f)

    return _open_file


@pytest.fixture(scope="function")
def open_raw_file():
    def _open_raw_file(name):
        with open(Path(__file__).parent.resolve() / "data" / name) as f:
            return f.read()

    return _open_raw_file


@pytest.fixture(scope="function")
def create_cve(app, open_file):
    def _create_cve(cve_id):
        CveUtil.create_cve(open_file(f"cves/{cve_id}.json"))
        return Cve.query.filter_by(cve_id=cve_id).first()

    return _create_cve


@pytest.fixture(scope="function")
def create_cves(create_cve):
    def _create_cves(cves):
        return [create_cve(cve) for cve in cves]

    return _create_cves


@pytest.fixture
def handle_events(app, open_file):
    def _handle_events(name):
        def _has_changed():
            current = Mock()
            current.value = Mock()
            current.value.return_value = "currentsha256"
            return current, "newsha256"

        def _download_modified():
            return open_file(name)

        with patch("opencve.tasks.events.has_changed") as mock:
            mock.return_value = _has_changed()

            with patch("opencve.tasks.events.download_modified_items") as mock:
                mock.return_value = _download_modified()
                handle_events_task()

    return _handle_events


@pytest.fixture
def create_user(app):
    def _create_user(name="user"):
        user = User(
            username=name,
            email=f"{name}@opencve.io",
            active=True,
            admin=False,
            email_confirmed_at=datetime.datetime.utcnow(),
            password=app.user_manager.hash_password("password"),
        )
        db.session.add(user)
        db.session.commit()
        return user

    return _create_user


@pytest.fixture
def create_vendor():
    def _create_vendor(vendor_name, product_name=None):
        vendor = Vendor.query.filter_by(name=vendor_name).first()
        if not vendor:
            vendor = Vendor(name=vendor_name)
        if product_name:
            vendor.products.append(Product(name=product_name))
        db.session.add(vendor)
        db.session.commit()
        return vendor

    return _create_vendor


@pytest.fixture(scope="function")
def create_cwe(app):
    def _create_cwe(cwe_id, name=None, description=None):
        cwe = Cwe(cwe_id=cwe_id, name=name, description=description)
        db.session.add(cwe)
        db.session.commit()
        return cwe

    return _create_cwe


@pytest.fixture(scope="function")
def create_cwes(create_cwe):
    def _create_cwes(cwes):
        return [create_cwe(cwe) for cwe in cwes]

    return _create_cwes


@pytest.fixture(scope="function")
def create_tag():
    def _create_tag(name, description, color, username="user"):
        user = User.query.filter_by(username=username).first()
        tag = UserTag(
            name=name,
            description=description,
            color=color,
            user=user,
        )
        db.session.add(tag)
        db.session.commit()
        return tag

    return _create_tag


@pytest.fixture
def make_soup():
    def _make_soup(data):
        return BeautifulSoup(data, "html.parser")

    return _make_soup


@pytest.fixture
def get_cve_names():
    def _get_cve_names(soup):
        return [
            cve.text
            for cve in soup.select("table#cves tr.cve-header td:first-child strong")
        ]

    return _get_cve_names


@pytest.fixture(scope="function")
def login(create_user, client):
    create_user()
    client.post(
        url_for("user.login"),
        data={"username": "user", "password": "password"},
        follow_redirects=True,
    )

    yield

    client.post(url_for("user.logout"), follow_redirects=True)



import arrow
from flask_restful import fields

from opencve.context import _humanize_filter
from opencve.utils import convert_cpes


class HumanizedNameField(fields.Raw):
    """
    Returns a humanized name.
    """

    def format(self, value):
        return _humanize_filter(value)


class ProductsListField(fields.Raw):
    """
    Returns a list of products.
    """

    def format(self, products):
        return sorted([product.name for product in products])


class CveVendorsField(fields.Raw):
    """
    Returns a list of vendors and products for a given CVE.
    """

    def format(self, json):
        return convert_cpes(json["configurations"])


class DatetimeField(fields.Raw):
    def format(self, value):
        """
        Returns UTC datetime.
        """
        return str(arrow.get(value).to("utc").strftime("%Y-%m-%dT%H:%M:%SZ"))



"""Add settings columns in users table

Revision ID: 3f755e0484e0
Revises: 2132d05ea0e2
Create Date: 2021-12-23 22:57:59.990718

"""

# revision identifiers, used by Alembic.
revision = "3f755e0484e0"
down_revision = "2132d05ea0e2"

from alembic import op
import sqlalchemy as sa
from sqlalchemy_utils import JSONType


def upgrade():
    op.add_column("users", sa.Column("settings", JSONType(), nullable=True))
    op.execute('UPDATE users SET settings = \'{"activities_view": "subscriptions"}\';')
    op.alter_column("users", "settings", existing_type=JSONType(), nullable=False)


def downgrade():
    op.drop_column("users", "settings")



from flask import request
from flask_restful import fields, marshal_with

from opencve.api.base import BaseResource
from opencve.api.fields import HumanizedNameField
from opencve.models.users import User


vendor_list_fields = {
    "name": fields.String(attribute="name"),
    "human_name": HumanizedNameField(attribute="name"),
}

product_list_fields = {
    "name": fields.String(attribute="name"),
    "human_name": HumanizedNameField(attribute="name"),
    "parent_vendor": fields.String(attribute="vendor.name"),
}


class SubscriptionListRessourceVendor(BaseResource):
    @marshal_with(vendor_list_fields)
    def get(self):
        user = User.query.filter_by(
            username=request.authorization.get("username")
        ).first()
        return user.vendors


class SubscriptionListRessourceProduct(BaseResource):
    @marshal_with(product_list_fields)
    def get(self):
        user = User.query.filter_by(
            username=request.authorization.get("username")
        ).first()
        return user.products



from opencve.extensions import db
from opencve.models import BaseModel


class Task(BaseModel):
    __tablename__ = "tasks"

    # Relationships
    changes = db.relationship("Change", back_populates="task")

    def __repr__(self):
        return "<Task {}>".format(self.created_at)



import os

from flask import Blueprint
from jinja2.filters import do_mark_safe

from opencve.configuration import OPENCVE_WELCOME_FILES
from opencve.constants import EMAIL_CONFIRMATION_MESSAGE
from opencve.context import (
    _cvss_bg,
    _cvss_percent,
    _excerpt,
    _event_description,
    _event_excerpt,
    _humanize_filter,
    _metric_bg,
    _remove_product_separator,
    _report_excerpt,
)

main = Blueprint(
    "main",
    __name__,
    static_folder=os.path.join(os.path.dirname(os.path.dirname(__file__)), "static"),
)

welcome = Blueprint(
    "welcome",
    __name__,
    static_url_path="/static/welcome",
    static_folder=OPENCVE_WELCOME_FILES,
    template_folder=OPENCVE_WELCOME_FILES,
)


@main.context_processor
def cvss_percent():
    return {"cvss_score_percent": _cvss_percent}


@main.context_processor
def cvss_bg():
    return {"cvss_bg": _cvss_bg}


@main.context_processor
def metric_bg():
    return {"metric_bg": _metric_bg}


from flask import url_for


def url_for_asset(filename):
    return url_for("static", filename=filename)


@main.context_processor
def get_url_for_asset():
    return {"url_for_asset": url_for_asset}


@main.app_template_filter("humanize")
def humanize_filter(s):
    return _humanize_filter(s)


@main.app_template_filter("vendors_excerpt")
def vendors_excerpt(s):
    return _excerpt(s, "vendors")


@main.app_template_filter("products_excerpt")
def products_excerpt(s):
    return _excerpt(s, "products")


@main.app_template_filter("tags_excerpt")
def tags_excerpt(s):
    return _excerpt(s, "tags")


@main.app_template_filter("report_excerpt")
def report_excerpt(s):
    return _report_excerpt(s)


@main.app_template_filter("event_excerpt")
def event_excerpt(s):
    return _event_excerpt(s)


@main.app_template_filter("event_description")
def event_description(s):
    return _event_description(s)


@main.app_template_filter("remove_product_separator")
def remove_product_separator(s):
    return _remove_product_separator(s)


@main.app_template_filter("custom_safe")
def custom_safe(s):
    if s == EMAIL_CONFIRMATION_MESSAGE:
        return do_mark_safe(s)
    return s



import json

from flask import current_app as app
from flask import abort, redirect, render_template, request, url_for
from flask_paginate import Pagination
from sqlalchemy import and_, or_

from opencve.constants import PRODUCT_SEPARATOR
from opencve.controllers.base import BaseController
from opencve.controllers.main import main
from opencve.controllers.tags import UserTagController
from opencve.models.cve import Cve
from opencve.models.products import Product
from opencve.models.tags import CveTag
from opencve.models.vendors import Vendor

from opencve.models.cwe import Cwe


class CveController(BaseController):
    model = Cve
    order = [Cve.updated_at.desc(), Cve.id.desc()]
    per_page_param = "CVES_PER_PAGE"
    schema = {
        "search": {"type": str},
        "vendor": {"type": str},
        "product": {"type": str},
        "cvss": {"type": str},
        "cwe": {"type": str},
        "tag": {"type": str},
        "user_id": {"type": str},
    }

    @classmethod
    def build_query(cls, args):
        vendor = None
        product = None
        tag = None
        query = Cve.query

        vendor_query = args.get("vendor")
        product_query = args.get("product")

        if vendor_query:
            vendor_query = vendor_query.replace(" ", "").lower()

        if product_query:
            product_query = product_query.replace(" ", "_").lower()

        # Filter by keyword
        if args.get("search"):

            possible_vendor = args.get("search").replace(" ", "").lower()
            possible_product = args.get("search").replace(" ", "_").lower()

            vendor = Vendor.query.filter_by(name=possible_vendor).first()

            if vendor:
                product = Product.query.filter_by(
                    name=possible_product, vendor_id=vendor.id
                ).first()
            else:
                product = Product.query.filter_by(name=possible_product).first()

            query = query.filter(
                or_(
                    Cve.cve_id.contains(args.get("search")),
                    Cve.summary.ilike(f"%{args.get('search')}%"),
                    Cve.vendors.contains([vendor.name]) if vendor else None,
                    Cve.vendors.contains([product.name]) if product else None,
                )
            )

        # Filter by CWE
        if args.get("cwe"):
            query = query.filter(Cve.cwes.contains([args.get("cwe")]))

        # Filter by CVSS score
        if args.get("cvss") and args.get("cvss").lower() in [
            "none",
            "low",
            "medium",
            "high",
            "critical",
        ]:
            if args.get("cvss").lower() == "none":
                query = query.filter(Cve.cvss3 == None)

            if args.get("cvss").lower() == "low":
                query = query.filter(and_(Cve.cvss3 >= 0.1, Cve.cvss3 <= 3.9))

            if args.get("cvss").lower() == "medium":
                query = query.filter(and_(Cve.cvss3 >= 4.0, Cve.cvss3 <= 6.9))

            if args.get("cvss").lower() == "high":
                query = query.filter(and_(Cve.cvss3 >= 7.0, Cve.cvss3 <= 8.9))

            if args.get("cvss").lower() == "critical":
                query = query.filter(and_(Cve.cvss3 >= 9.0, Cve.cvss3 <= 10.0))

        # Filter by vendor and product
        if vendor_query and product_query:
            vendor = Vendor.query.filter_by(name=vendor_query).first()
            if not vendor:
                abort(404, "Not found.")

            product = Product.query.filter_by(
                name=product_query, vendor_id=vendor.id
            ).first()
            if not product:
                abort(404, "Not found.")

            query = query.filter(
                Cve.vendors.contains(
                    [f"{vendor.name}{PRODUCT_SEPARATOR}{product.name}"]
                )
            )

        # Filter by vendor
        elif vendor_query:
            vendor = Vendor.query.filter_by(name=vendor_query).first()
            if not vendor:
                abort(404, "Not found.")
            query = query.filter(Cve.vendors.contains([vendor.name]))

        # Filter by product only
        elif product_query:
            product = Product.query.filter_by(name=product_query).first()
            if not product:
                abort(404, "Not found.")
            query = query.filter(Cve.vendors.contains([product.name]))

        # Filter by tag
        if args.get("tag"):
            tag = UserTagController.get(
                {"user_id": args.get("user_id"), "name": args.get("tag")}
            )
            if not tag:
                abort(404, "Not found.")
            query = (
                query.join(CveTag)
                .filter(CveTag.user_id == args.get("user_id"))
                .filter(CveTag.tags.contains([args.get("tag")]))
            )

        return query, {"vendor": vendor, "product": product, "tag": tag}



from opencve.commands.create_user import create_user
from opencve.models.users import User


def test_create_user_no_parameters(app):
    runner = app.test_cli_runner()
    result = runner.invoke(create_user, [])

    assert result.exit_code == 2
    assert "Usage: create-user [OPTIONS] USERNAME EMAIL" in result.output


def test_create_user_wrong_parameters(app):
    runner = app.test_cli_runner()

    result = runner.invoke(create_user, ["john"])
    assert result.exit_code == 2
    assert "Missing argument 'EMAIL'" in result.output

    result = runner.invoke(create_user, ["john", "john@domain.com", "--foo"])
    assert result.exit_code == 2
    assert "Error: no such option: --foo" in result.output


def test_create_simple_user(app):
    runner = app.test_cli_runner()

    result = runner.invoke(
        create_user, ["john", "john@domain.com"], input="password\npassword\n"
    )
    assert result.exit_code == 0
    assert "[*] User john created." in result.output

    user = User.query.first()
    assert user.username == "john"
    assert user.email == "john@domain.com"
    assert not user.admin


def test_create_admin(app):
    runner = app.test_cli_runner()

    result = runner.invoke(
        create_user,
        ["john", "john@domain.com", "--admin"],
        input="password\npassword\n",
    )
    assert result.exit_code == 0
    assert "[*] User john created." in result.output

    user = User.query.first()
    assert user.username == "john"
    assert user.email == "john@domain.com"
    assert user.admin


def test_create_user_already_exists(app):
    runner = app.test_cli_runner()

    result = runner.invoke(
        create_user, ["john", "john@domain.com"], input="password\npassword\n"
    )
    assert result.exit_code == 0
    assert "[*] User john created." in result.output

    result = runner.invoke(
        create_user, ["john", "john@domain.com"], input="password\npassword\n"
    )
    assert result.exit_code == 2
    assert "Error: Invalid value for username: john already exists." in result.output

    result = runner.invoke(
        create_user, ["john2", "john@domain.com"], input="password\npassword\n"
    )
    assert result.exit_code == 2
    assert (
        "Error: Invalid value for email: john@domain.com already exists."
        in result.output
    )



from unittest.mock import patch

from opencve.commands.celery import beat, worker


@patch("opencve.commands.webserver.os.execvp")
def test_celery_worker_command(mock_execvp, app):
    mock_execvp.return_value = True
    runner = app.test_cli_runner()

    # Without arguments
    runner.invoke(worker, [])
    mock_execvp.assert_called_with(
        "celery", ["celery", "worker", "-A", "opencve.app:cel"]
    )

    # With custom arguments
    runner.invoke(worker, ["-l", "info"])
    mock_execvp.assert_called_with(
        "celery", ["celery", "worker", "-A", "opencve.app:cel", "-l", "info"]
    )


@patch("opencve.commands.webserver.os.execvp")
def test_celery_beat_command(mock_execvp, app):
    mock_execvp.return_value = True
    runner = app.test_cli_runner()

    # Without arguments
    runner.invoke(beat, [])
    mock_execvp.assert_called_with(
        "celery", ["celery", "beat", "-A", "opencve.app:cel"]
    )

    # With custom arguments
    runner.invoke(beat, ["-l", "info"])
    mock_execvp.assert_called_with(
        "celery", ["celery", "beat", "-A", "opencve.app:cel", "-l", "info"]
    )



from opencve.constants import PRODUCT_SEPARATOR
from opencve.extensions import db
from opencve.utils import convert_cpes, flatten_vendors, get_cwes, get_cwes_details
from opencve.models.cwe import Cwe


def test_convert_empty_conf():
    assert convert_cpes({}) == {}


def test_convert_simple_conf(open_file):
    conf = open_file("configurations/simple.json")
    assert convert_cpes(conf) == {"foo": ["bar"]}


def test_convert_conf_multiple_products(open_file):
    conf = open_file("configurations/multiple_products.json")
    vendors = convert_cpes(conf)
    vendors["foo"] = sorted(vendors["foo"])
    assert vendors == {"foo": ["bar", "baz"]}


def test_convert_conf_multiple_vendors(open_file):
    conf = open_file("configurations/multiple_vendors.json")
    vendors = convert_cpes(conf)
    vendors["foo"] = sorted(vendors["foo"])
    assert len(vendors) == 2
    assert vendors["foo"] == ["bar", "baz"]
    assert vendors["bar"] == ["baz"]


def test_convert_conf_nested(open_file):
    conf = open_file("configurations/nested.json")
    vendors = convert_cpes(conf)
    vendors["foo"] = sorted(vendors["foo"])
    assert vendors == {"foo": ["bar", "baz"], "bar": ["baz"]}


def test_flatten_empty_vendors():
    assert flatten_vendors({}) == []


def test_flatten_simple_vendor():
    assert flatten_vendors({"foo": ["bar"]}) == ["foo", f"foo{PRODUCT_SEPARATOR}bar"]


def test_flatten_multiple_products():
    assert flatten_vendors({"foo": ["bar", "baz"]}) == [
        "foo",
        f"foo{PRODUCT_SEPARATOR}bar",
        f"foo{PRODUCT_SEPARATOR}baz",
    ]


def test_flatten_multiple_vendors():
    assert flatten_vendors({"foo": ["bar", "baz"], "bar": ["baz"]}) == [
        "foo",
        f"foo{PRODUCT_SEPARATOR}bar",
        f"foo{PRODUCT_SEPARATOR}baz",
        "bar",
        f"bar{PRODUCT_SEPARATOR}baz",
    ]


def test_get_cwes():
    problems = [
        {"lang": "en", "value": "CWE-732"},
        {"lang": "en", "value": "CWE-311"},
        {"lang": "en", "value": "CWE-532"},
    ]
    assert sorted(get_cwes(problems)) == ["CWE-311", "CWE-532", "CWE-732"]

    problems = []
    assert sorted(get_cwes(problems)) == []

    problems = [
        {"lang": "en", "value": "CWE-732"},
        {"lang": "en", "value": "CWE-732"},
        {"lang": "en", "value": "CWE-532"},
    ]
    assert sorted(get_cwes(problems)) == ["CWE-532", "CWE-732"]


def test_get_cwes_details():
    db.session.add(
        Cwe(cwe_id="CWE-1", name="Name of CWE-1", description="Description of CWE-1")
    )
    db.session.add(
        Cwe(cwe_id="CWE-2", name="Name of CWE-2", description="Description of CWE-2")
    )
    db.session.commit()

    cwes = get_cwes_details(
        [{"lang": "en", "value": "CWE-1"}, {"lang": "en", "value": "CWE-2"}]
    )
    assert cwes == {"CWE-1": "Name of CWE-1", "CWE-2": "Name of CWE-2"}

    cwes = get_cwes_details(
        [
            {"lang": "en", "value": "CWE-1"},
            {"lang": "en", "value": "CWE-1"},
            {"lang": "en", "value": "CWE-2"},
        ]
    )
    assert cwes == {"CWE-1": "Name of CWE-1", "CWE-2": "Name of CWE-2"}

    cwes = get_cwes_details([{"lang": "en", "value": "CWE-3"}])
    assert cwes == {"CWE-3": None}



from flask import current_app as app
from flask_user import UserMixin
from sqlalchemy.sql import expression
from sqlalchemy_utils import ChoiceType, JSONType

from opencve.constants import FREQUENCIES_TYPES
from opencve.extensions import db
from opencve.models import BaseModel, users_products, users_vendors


def get_default_filters():
    return {
        "cvss": 0,
        "event_types": [
            "new_cve",
            "first_time",
            "references",
            "cvss",
            "cpes",
            "summary",
            "cwes",
        ],
    }


def get_default_settings():
    return {"activities_view": "all"}


class User(BaseModel, UserMixin):
    __tablename__ = "users"
    __hash__ = UserMixin.__hash__

    # User authentication information
    username = db.Column(db.String(50), nullable=False, unique=True)
    password = db.Column(db.String(255), nullable=False, server_default="")
    reset_password_token = db.Column(db.String(100), nullable=False, server_default="")

    # User email information
    email = db.Column(db.String(255), nullable=False, unique=True)
    email_confirmed_at = db.Column(db.DateTime(timezone=True))

    # Notification parameters
    enable_notifications = db.Column(
        db.Boolean(), nullable=False, server_default=expression.true()
    )
    filters_notifications = db.Column(JSONType, default=get_default_filters)
    settings = db.Column(JSONType, default=get_default_settings, nullable=False)
    frequency_notifications = db.Column(ChoiceType(FREQUENCIES_TYPES), default="always")

    # User information
    active = db.Column(
        "is_active", db.Boolean(), nullable=False, server_default=expression.false()
    )
    first_name = db.Column(db.String(100), nullable=False, server_default="")
    last_name = db.Column(db.String(100), nullable=False, server_default="")
    admin = db.Column(db.Boolean, unique=False, server_default=expression.false())

    # Relationships
    vendors = db.relationship("Vendor", secondary=users_vendors)
    products = db.relationship("Product", secondary=users_products)
    alerts = db.relationship("Alert", back_populates="user")
    reports = db.relationship("Report", back_populates="user")
    tags = db.relationship("UserTag", back_populates="user")
    cve_tags = db.relationship("CveTag", back_populates="user")

    @property
    def is_confirmed(self):
        return bool(self.email_confirmed_at)

    def __repr__(self):
        return "<User {}>".format(self.username)

    def __eq__(self, user):
        return self.id == user.id if user else False

    def set_password(self, password):
        self.password = app.user_manager.hash_password(password)



import pytest
from werkzeug.exceptions import NotFound

from opencve.controllers.tags import UserTagController
from opencve.extensions import db
from opencve.models.tags import UserTag


def test_metas(app):
    with app.test_request_context():
        _, metas, _ = UserTagController.list()
    assert metas == {}


def test_list(app, create_user):
    user1 = create_user("user1")
    user1.tags.append(
        UserTag(
            name="mytag",
            description="my description",
            color="#fff",
            user=user1,
        )
    )
    db.session.commit()

    # User1 has one tag
    with app.test_request_context():
        tags = UserTagController.list_items({"user_id": user1.id})

    assert len(tags) == 1
    assert tags[0].user.id == user1.id
    assert tags[0].name == "mytag"
    assert tags[0].description == "my description"
    assert tags[0].color == "#fff"

    # User2 has no tag
    user2 = create_user("user2")
    with app.test_request_context():
        tags = UserTagController.list_items({"user_id": user2.id})

    assert len(tags) == 0


def test_list_paginated(app, create_user):
    user = create_user()

    # Change the pagination to 3 items
    old = app.config["TAGS_PER_PAGE"]
    app.config["TAGS_PER_PAGE"] = 3

    # Create 5 tags for the user
    for i in range(5):
        tag = UserTag(
            name=f"tag{i}",
            description="my description",
            color="#fff",
            user=user,
        )
        db.session.add(tag)
    db.session.commit()

    with app.test_request_context():
        tags = UserTagController.list_items({"user_id": user.id})
        assert len(tags) == 3
        assert [t.name for t in tags] == ["tag0", "tag1", "tag2"]

        tags = UserTagController.list_items({"user_id": user.id, "page": 2})
        assert len(tags) == 2
        assert [t.name for t in tags] == ["tag3", "tag4"]

        with pytest.raises(NotFound):
            UserTagController.list_items({"user_id": user.id, "page": 3})

    app.config["TAGS_PER_PAGE"] = old


def test_get(app, create_user):
    user = create_user()
    user.tags.append(
        UserTag(
            name="mytag",
            description="my description",
            color="#fff",
            user=user,
        )
    )
    db.session.commit()

    with app.test_request_context():
        tag = UserTagController.get({"user_id": user.id, "name": "mytag"})
        assert tag.name == "mytag"
        assert tag.user.id == user.id

        with pytest.raises(NotFound):
            UserTagController.get({"user_id": user.id, "name": "notfound"})



from flask import request

from opencve.extensions import db
from opencve.models.cve import Cve
from opencve.models.tags import UserTag
from opencve.models.users import User


def test_redirect_auth(client):
    response = client.get("/account/tags")
    assert response.status_code == 302

    with client:
        response = client.get("/account/tags", follow_redirects=True)
        assert response.status_code == 200
        assert request.path == "/login"


def test_no_tags(client, login):
    response = client.get("/account/tags")
    assert response.status_code == 200
    assert b"You have no tag yet." in response.data


def test_list_tags(client, login):
    user = User.query.first()

    # Create 5 tags for the user
    for i in range(5):
        tag = UserTag(
            name=f"tag{i}",
            description="my description",
            color="#fff",
            user=user,
        )
        db.session.add(tag)
    db.session.commit()

    response = client.get("/account/tags")
    assert response.status_code == 200
    assert b"tag0" in response.data
    assert b"tag1" in response.data
    assert b"tag2" in response.data
    assert b"tag3" in response.data
    assert b"tag4" in response.data


def test_create_tag(client, login):
    response = client.post(
        "/account/tags",
        data={"name": "tag1", "description": "my description", "color": "#ffffff"},
    )
    assert response.status_code == 302

    tag = UserTag.query.first()
    assert tag.name == "tag1"
    assert tag.description == "my description"
    assert tag.color == "#ffffff"

    with client:
        response = client.post(
            "/account/tags",
            data={"name": "tag2", "description": "my description", "color": "#ffffff"},
            follow_redirects=True,
        )
        assert response.status_code == 200
        assert b"The tag tag2 has been successfully added." in response.data
        assert b"tag1" in response.data
        assert b"tag2" in response.data


def test_create_tag_invalid_form(client, login):
    response = client.post("/account/tags", data={})
    assert b"Name is required" in response.data

    response = client.post(
        "/account/tags",
        data={"name": "my tag", "color": "not_valid"},
    )
    assert b"Color must be in hexadecimal format" in response.data


def test_create_existing_tag(client, login):
    response = client.post(
        "/account/tags",
        data={"name": "tag1", "description": "my description", "color": "#ffffff"},
        follow_redirects=True,
    )
    assert b"The tag tag1 has been successfully added." in response.data

    response = client.post(
        "/account/tags",
        data={"name": "tag1", "description": "my description", "color": "#ffffff"},
        follow_redirects=True,
    )
    assert b"This tag already exists." in response.data


def test_update_tag(client, login):
    response = client.post(
        "/account/tags",
        data={
            "name": "tag1",
            "description": "my original description",
            "color": "#ffffff",
        },
        follow_redirects=True,
    )
    assert b"my original description" in response.data

    response = client.post(
        "/account/tags/tag1",
        data={"description": "my edited description", "color": "#000000"},
        follow_redirects=True,
    )
    assert not b"my original description" in response.data
    assert b"my edited description" in response.data

    tag = UserTag.query.first()
    assert tag.name == "tag1"
    assert tag.description == "my edited description"
    assert tag.color == "#000000"


def test_delete_tag(client, login):
    response = client.get("/account/tags/tag1/delete")
    assert response.status_code == 404

    client.post(
        "/account/tags",
        data={
            "name": "tag1",
            "description": "my original description",
            "color": "#ffffff",
        },
    )

    response = client.get("/account/tags/tag1/delete")
    assert response.status_code == 200
    assert (
        b"Do you really want to delete the <strong>tag1</strong> tag ?" in response.data
    )

    response = client.post("/account/tags/tag1/delete", follow_redirects=True)
    assert b"The tag tag1 has been deleted." in response.data
    assert UserTag.query.count() == 0


def test_associate_tags(client, login, create_cve, create_tag):
    create_cve("CVE-2018-18074")
    create_tag("tag1", "my description", "#ffffff")
    create_tag("tag2", "my description", "#ffffff")

    with client:
        # The CVE has no tag
        response = client.get("/cve/CVE-2018-18074")
        assert not b">tag1</span>" in response.data
        assert not b">tag2</span>" in response.data
        cve = Cve.query.filter_by(cve_id="CVE-2018-18074").first()
        assert not cve.tags

        # CVE not found
        response = client.post("/cve/CVE-2000-0001/tags", data={})
        assert response.status_code == 404

        # Tag not found
        response = client.post("/cve/CVE-2018-18074/tags", data={"tags": ["notfound"]})
        assert response.status_code == 404

        # Associate tags
        response = client.post(
            "/cve/CVE-2018-18074/tags",
            data={"tags": ["tag1", "tag2"]},
            follow_redirects=True,
        )
        assert b"The CVE tags have been updated." in response.data
        assert b">tag1</span>" in response.data
        assert b">tag2</span>" in response.data
        cve = Cve.query.filter_by(cve_id="CVE-2018-18074").first()
        assert len(cve.tags) == 2
        assert [t.name for t in cve.tags] == ["tag1", "tag2"]

        # Remove one tag
        response = client.post(
            "/cve/CVE-2018-18074/tags", data={"tags": ["tag1"]}, follow_redirects=True
        )
        assert b"The CVE tags have been updated." in response.data
        assert b">tag1</span>" in response.data
        assert not b">tag2</span>" in response.data
        cve = Cve.query.filter_by(cve_id="CVE-2018-18074").first()
        assert len(cve.tags) == 1
        assert cve.tags[0].name == "tag1"

        # Remove all tags
        response = client.post(
            "/cve/CVE-2018-18074/tags", data={}, follow_redirects=True
        )
        assert b"The CVE tags have been updated." in response.data
        assert not b">tag1</span>" in response.data
        assert not b">tag2</span>" in response.data
        cve = Cve.query.filter_by(cve_id="CVE-2018-18074").first()
        assert len(cve.tags) == 0



import pytest
from flask import request

from opencve.models.users import User


@pytest.mark.parametrize(
    "first_name,last_name,email",
    [
        ("john", "doe", "john.doe@example.com"),
        ("john", "", "john.doe@example.com"),
        ("", "doe", "john.doe@example.com"),
        ("", "", "john.doe@example.com"),
        ("", "", "user@opencve.io"),
    ],
)
def test_edit_profile(login, client, first_name, last_name, email):
    user = User.query.first()
    assert user.first_name == ""
    assert user.last_name == ""
    assert user.email == "user@opencve.io"

    client.post(
        "/account/profile",
        data={"first_name": first_name, "last_name": last_name, "email": email},
        follow_redirects=True,
    )

    user = User.query.first()
    assert user.first_name == first_name
    assert user.last_name == last_name
    assert user.email == email


def test_edit_profile_email_required(login, client):
    response = client.post(
        "/account/profile", data={"email": ""}, follow_redirects=True
    )
    assert b"This field is required." in response.data


def test_edit_profile_with_existing_email(login, create_user, client):
    user = User.query.filter_by(username="user").first()
    assert user.email == "user@opencve.io"

    create_user("user2")

    response = client.post(
        "/account/profile", data={"email": "user2@opencve.io"}, follow_redirects=True
    )
    assert b"This Email is already in use. Please try another one" in response.data

    # Email has not changed
    user = User.query.filter_by(username="user").first()
    assert user.email == "user@opencve.io"



from celery.utils.log import get_task_logger

from opencve.constants import PRODUCT_SEPARATOR
from opencve.extensions import cel, db
from opencve.models.alerts import Alert
from opencve.models.cve import Cve
from opencve.models.events import Event
from opencve.models.products import Product
from opencve.models.vendors import Vendor

logger = get_task_logger(__name__)


def filter_events(user, events):
    # Only keep the wanted events
    filtered_events = {
        e.type.code: e
        for e in events
        if e.type.code in user.filters_notifications["event_types"]
    }

    # Check if new vendors/products match the user's subscriptions
    if "first_time" in filtered_events.keys():

        # TODO: refactor with controllers.home::home (+tests)
        subscriptions = [v.name for v in user.vendors]
        subscriptions.extend(
            [f"{p.vendor.name}{PRODUCT_SEPARATOR}{p.name}" for p in user.products]
        )

        if not any(s in filtered_events["first_time"].details for s in subscriptions):
            del filtered_events["first_time"]

    return list(filtered_events.values())


@cel.task(name="HANDLE_ALERTS")
def handle_alerts():
    cel.app.app_context().push()

    logger.info("Checking for new alerts...")

    # Retrieve the CVE list for which events no reviewed exist.
    cves = Cve.query.filter(Cve.events.any(Event.review == False)).all()
    if not cves:
        logger.info("No CVE to review")
        return

    # Check each CVE, get its events and create alerts
    logger.info("Checking {} CVE containing event(s) no reviewed...".format(len(cves)))

    for cve in cves:
        users = {}
        events = Event.query.filter_by(cve=cve, review=False).all()
        logger.info(
            "{} contains {} events to review...".format(cve.cve_id, len(events))
        )

        # Save the subscribers for each vendor of the CVE
        for v in cve.vendors:

            # Product contains the separator
            if PRODUCT_SEPARATOR in v:
                vendor = Vendor.query.filter_by(
                    name=v.split(PRODUCT_SEPARATOR)[0]
                ).first()
                product = Product.query.filter_by(
                    name=v.split(PRODUCT_SEPARATOR)[1], vendor_id=vendor.id
                ).first()
                for user in product.users:
                    if user not in users.keys():
                        users[user] = {"products": [], "vendors": []}
                    users[user]["products"].append(product.name)

            # Vendor
            else:
                vendor = Vendor.query.filter_by(name=v).first()
                for user in vendor.users:
                    if user not in users.keys():
                        users[user] = {"products": [], "vendors": []}
                    users[user]["vendors"].append(vendor.name)

        # No users concerned
        if not users:
            logger.info("No users to alert.")
            for event in events:
                event.review = True
            db.session.commit()
            continue

        # Users need to be alerted
        logger.info("{} users found, creating the alerts...".format(len(users)))

        for user, details in users.items():

            # Filter by CVSS v3 score
            cvss_score = cve.cvss3

            if cvss_score and cvss_score < user.filters_notifications["cvss"]:
                logger.info(
                    "Skip alert for {0} because of CVSSv3 filter ({1} < {2})".format(
                        user.username, cvss_score, user.filters_notifications["cvss"]
                    )
                )
                continue

            # Keep the wanted filter by user
            events_copy = list(events)
            events_copy = filter_events(user, events_copy)

            if not events_copy:
                logger.info(
                    "No event matches the filters for {0}".format(user.username)
                )
            else:
                logger.info(
                    "Events match for {0} ({1})".format(
                        user.username, ",".join(e.type.code for e in events_copy)
                    )
                )

                # We add the filters in the details
                details["filters"] = [e.type.code for e in events_copy]

                # An alert is composed of a CVE, events for that CVE,
                # and details including vendors and products.
                alert = Alert(
                    user=user, cve=cve, details=details, events=events, notify=False
                )
                db.session.add(alert)
                db.session.commit()

                logger.info(
                    "Alert created for {} (ID: {})".format(user.username, alert.id)
                )

        # We can review the events
        for event in events:
            event.review = True
        db.session.commit()



from collections import OrderedDict
from datetime import datetime, time

import arrow
from celery.utils.log import get_task_logger
from flask import render_template
from flask_user import EmailError
from sqlalchemy import delete

from opencve.context import _humanize_filter
from opencve.extensions import cel, db, user_manager
from opencve.models.alerts import Alert
from opencve.models.cve import Cve
from opencve.models.reports import Report
from opencve.models.users import User

logger = get_task_logger(__name__)


def get_users_with_alerts():
    """
    If we are between 11:00 AM and 11:15 AM, we get all the users. Otherwise
    we only select the 'always' frequency ones (todo: find a cleaner solution).
    """
    now = datetime.now()
    query = User.query.filter(User.alerts.any(Alert.notify == False))

    if time(11, 0) <= now.time() <= time(11, 15):
        logger.info("We are between 11:00 AM and 11:15 AM, get all the users...")
        users = query.all()
    else:
        logger.info("Get the users who want to always receive email...")
        users = query.filter(User.frequency_notifications == "always").all()

    return users


def get_top_alerts(user, count=10):
    """
    Return the top X alerts for a given user.
    """
    top_alerts = (
        db.session.query(Alert.id)
        .filter_by(user=user, notify=False)
        .join(Alert.cve)
        .order_by(Cve.cvss3.desc())
        .limit(count)
        .all()
    )

    # Convert this list of ID in a list of objects
    top_alerts = [alert[0] for alert in top_alerts]
    top_alerts = db.session.query(Alert).filter(Alert.id.in_(top_alerts)).all()

    return top_alerts


def get_sorted_alerts(alerts):
    """
    Sort the alerts by vendors and products then extract their max score.
    """
    alerts_sorted = {}

    for alert in alerts:
        for vendor in alert.details["vendors"]:
            if vendor not in alerts_sorted:
                alerts_sorted[vendor] = {
                    "name": _humanize_filter(vendor),
                    "alerts": [],
                    "max": 0,
                }
            alerts_sorted[vendor]["alerts"].append(alert)
        for product in alert.details["products"]:
            if product not in alerts_sorted:
                alerts_sorted[product] = {
                    "name": _humanize_filter(product),
                    "alerts": [],
                    "max": 0,
                }
            alerts_sorted[product]["alerts"].append(alert)

    # For each vendor, we take the max score
    for k, als in alerts_sorted.items():

        # Get the max score
        cvss = [al.cve.cvss3 for al in als["alerts"] if al.cve.cvss3]
        if cvss:
            alerts_sorted[k]["max"] = max(cvss)

    alerts_sorted = OrderedDict(
        sorted(alerts_sorted.items(), key=lambda i: i[1]["max"], reverse=True)
    )

    return alerts_sorted


def get_vendors_products(alerts):
    """
    Returns a sorted list of vendors given some alerts.
    """
    vendors_products = []
    for alert in alerts:
        vendors_products.extend(
            sorted(
                list(set(alert.details["vendors"]))
                + list(set(alert.details["products"]))
            )
        )

    # Remove duplicates
    vendors_products = sorted(list(set(vendors_products)))

    return vendors_products


@cel.task(name="HANDLE_REPORTS")
def handle_reports():
    cel.app.app_context().push()

    # The server name is mandatory to generate the mails
    if not cel.app.config.get("SERVER_NAME"):
        raise ValueError(
            "The `server_name` variable is not set in your `opencve.cf` file. "
            "Please configure it to allow OpenCVE to create reports and send the mails."
        )

    # Get users to nofity
    users = get_users_with_alerts()
    if not users:
        logger.info("No alert to send.")
        return

    # Get alerts for all users, create a report containing it
    # and send a mail with the top alerts.
    logger.info("Checking {} users with alerts to send...".format(len(users)))

    for user in users:
        alerts = Alert.query.filter_by(user=user, notify=False).all()
        logger.info("{} alerts to notify for {}".format(len(alerts), user.username))

        top_alerts = get_top_alerts(user)
        sorted_alerts = get_sorted_alerts(top_alerts)
        all_vendors_products = get_vendors_products(alerts)

        # Create the report
        report = Report(user=user, alerts=alerts, details=all_vendors_products)
        db.session.add(report)
        db.session.commit()

        logger.info("Report {0} created.".format(report.id))

        if not user.enable_notifications:
            logger.info(
                "User {} do not want to receive email notifications, skip it.".format(
                    user.username
                )
            )
        else:
            alert_str = "alerts" if len(alerts) > 1 else "alert"
            subject = "{count} {alerts} on {vendors}".format(
                count=len(alerts),
                alerts=alert_str,
                vendors=", ".join(list(map(_humanize_filter, all_vendors_products))),
            )
            try:
                user_manager.email_manager.send_user_report(
                    user,
                    **{
                        "subject": subject,
                        "total_alerts": len(alerts),
                        "alerts_sorted": sorted_alerts,
                        "report_public_link": report.public_link,
                    },
                )
                logger.info("Mail sent for {}".format(user.email))
            except EmailError as e:
                logger.error(f"EmailError : {e}")

        # The alerts have been notified
        for alert in alerts:
            alert.notify = True
        db.session.commit()


@cel.task(name="REPORTS_CLEANUP")
def reports_cleanup():
    cel.app.app_context().push()

    if not cel.app.config.get("REPORTS_CLEANUP_DAYS"):
        return

    # Remove N days to the current date
    nb_days = -abs(cel.app.config.get("REPORTS_CLEANUP_DAYS"))
    shifted_date = arrow.utcnow().shift(days=nb_days)
    logger.info(f"Removing old reports ({shifted_date.humanize()})...")

    # Use SQLAlchemy core for performance
    stmt = delete(Report).where(Report.created_at < shifted_date.datetime)
    db.engine.execute(stmt)



from flask import request, render_template

from opencve.controllers.main import main
from opencve.controllers.products import ProductController
from opencve.controllers.vendors import VendorController


@main.route("/vendors")
def vendors():
    vendors, _, pagination_v = VendorController.list(request.args)
    products, _, pagination_p = ProductController.list(request.args)

    return render_template(
        "vendors.html",
        vendors=vendors,
        products=products,
        pagination_v=pagination_v,
        pagination_p=pagination_p,
    )



import itertools
import json
import operator

from flask import abort, flash, redirect, request, render_template, url_for
from flask_user import current_user, login_required

from opencve.controllers.cves import CveController
from opencve.controllers.main import main
from opencve.controllers.tags import UserTagController
from opencve.extensions import db
from opencve.models import is_valid_uuid
from opencve.models.changes import Change
from opencve.models.events import Event
from opencve.models.tags import CveTag
from opencve.utils import convert_cpes, get_cwes_details, CustomHtmlHTML


@main.route("/cve")
def cves():
    args = request.args
    user_tags = []
    if current_user.is_authenticated:
        args = {**request.args, "user_id": current_user.id}
        user_tags = UserTagController.list_items({"user_id": current_user.id})

    objects, metas, pagination = CveController.list(args)

    return render_template(
        "cves.html",
        cves=objects,
        vendor=metas.get("vendor"),
        product=metas.get("product"),
        tag=metas.get("tag"),
        user_tags=user_tags,
        pagination=pagination,
    )


@main.route("/cve/<cve_id>")
def cve(cve_id):
    cve = CveController.get({"cve_id": cve_id})

    vendors = convert_cpes(cve.json["configurations"])
    cwes = get_cwes_details(
        cve.json["cve"]["problemtype"]["problemtype_data"][0]["description"]
    )

    # Get the user tags
    user_tags = []
    if current_user.is_authenticated:
        user_tags = UserTagController.list_items({"user_id": current_user.id})

    # We have to pass an encoded list of tags for the modal box
    cve_tags_encoded = json.dumps([t.name for t in cve.tags])

    events = Event.query.filter_by(cve_id=cve.id).order_by(Event.created_at.desc())

    events_by_time = [
        (time, list(evs))
        for time, evs in (itertools.groupby(events, operator.attrgetter("created_at")))
    ]

    return render_template(
        "cve.html",
        cve=cve,
        cve_dumped=json.dumps(cve.json),
        vendors=vendors,
        cwes=cwes,
        user_tags=user_tags,
        cve_tags_encoded=cve_tags_encoded,
        events_by_time=events_by_time,
    )


@main.route("/cve/<cve_id>/tags", methods=["POST"])
@login_required
def cve_associate_tags(cve_id):
    cve = CveController.get({"cve_id": cve_id})
    new_tags = request.form.getlist("tags")

    # Check if all tags are declared by the user
    user_tags = [
        t.name for t in UserTagController.list_items({"user_id": current_user.id})
    ]
    for new_tag in new_tags:
        if new_tag not in user_tags:
            abort(404)

    # Update the CVE tags
    cve_tag = CveTag.query.filter_by(user_id=current_user.id, cve_id=cve.id).first()

    if not cve_tag:
        cve_tag = CveTag(user_id=current_user.id, cve_id=cve.id)

    cve_tag.tags = new_tags
    db.session.add(cve_tag)
    db.session.commit()

    flash("The CVE tags have been updated.", "success")
    return redirect(url_for("main.cve", cve_id=cve_id))


@main.route("/cve/<cve_id>/changes/<change_id>")
def cve_change(cve_id, change_id):
    cve = CveController.get({"cve_id": cve_id})

    if not is_valid_uuid(change_id):
        abort(404)

    change = Change.query.filter_by(cve_id=cve.id, id=change_id).first()
    if not change:
        abort(404)

    previous = (
        Change.query.filter(Change.created_at < change.created_at)
        .filter(Change.cve == change.cve)
        .order_by(Change.created_at.desc())
        .first()
    )

    previous_json = {}
    if previous:
        previous_json = previous.json

    differ = CustomHtmlHTML()
    diff = differ.make_table(
        fromlines=json.dumps(previous_json, sort_keys=True, indent=2).split("\n"),
        tolines=json.dumps(change.json, sort_keys=True, indent=2).split("\n"),
        context=True,
    )

    return render_template("change.html", change=change, diff=diff)



"""Initial migration

Revision ID: 8bdc527d8d49
Revises: None
Create Date: 2020-03-28 13:06:42.037072

"""

# revision identifiers, used by Alembic.
revision = "8bdc527d8d49"
down_revision = None

from alembic import op
import sqlalchemy as sa
from sqlalchemy_utils import UUIDType, JSONType
from sqlalchemy.dialects import postgresql


def upgrade():
    op.create_table(
        "cves",
        sa.Column("id", UUIDType(binary=False), nullable=False),
        sa.Column("created_at", sa.DateTime(timezone=True), nullable=False),
        sa.Column("updated_at", sa.DateTime(timezone=True), nullable=False),
        sa.Column("cve_id", sa.String(), nullable=False),
        sa.Column("json", postgresql.JSONB(astext_type=sa.Text()), nullable=True),
        sa.Column("vendors", postgresql.JSONB(astext_type=sa.Text()), nullable=True),
        sa.Column("cwes", postgresql.JSONB(astext_type=sa.Text()), nullable=True),
        sa.Column("summary", sa.String(), nullable=False),
        sa.Column("cvss2", sa.Float(), nullable=True),
        sa.Column("cvss3", sa.Float(), nullable=True),
        sa.PrimaryKeyConstraint("id"),
    )
    op.create_index(op.f("ix_cves_created_at"), "cves", ["created_at"], unique=False)
    op.create_index(op.f("ix_cves_cve_id"), "cves", ["cve_id"], unique=False)
    op.create_index(op.f("ix_cves_updated_at"), "cves", ["updated_at"], unique=False)
    op.create_index(
        "ix_cves_vendors",
        "cves",
        ["vendors"],
        unique=False,
        postgresql_using="gin",
    )
    op.create_index(
        "ix_cves_cwes", "cves", ["cwes"], unique=False, postgresql_using="gin"
    )
    op.create_table(
        "cwes",
        sa.Column("id", UUIDType(binary=False), nullable=False),
        sa.Column("created_at", sa.DateTime(timezone=True), nullable=False),
        sa.Column("updated_at", sa.DateTime(timezone=True), nullable=False),
        sa.Column("cwe_id", sa.String(), nullable=True),
        sa.Column("name", sa.String(), nullable=True),
        sa.Column("description", sa.String(), nullable=True),
        sa.PrimaryKeyConstraint("id"),
    )
    op.create_index(op.f("ix_cwes_created_at"), "cwes", ["created_at"], unique=False)
    op.create_table(
        "metas",
        sa.Column("id", UUIDType(binary=False), nullable=False),
        sa.Column("created_at", sa.DateTime(timezone=True), nullable=False),
        sa.Column("updated_at", sa.DateTime(timezone=True), nullable=False),
        sa.Column("name", sa.String(), nullable=False),
        sa.Column("value", sa.String(), nullable=False),
        sa.PrimaryKeyConstraint("id"),
    )
    op.create_index(op.f("ix_metas_created_at"), "metas", ["created_at"], unique=False)
    op.create_table(
        "tasks",
        sa.Column("id", UUIDType(binary=False), nullable=False),
        sa.Column("created_at", sa.DateTime(timezone=True), nullable=False),
        sa.Column("updated_at", sa.DateTime(timezone=True), nullable=False),
        sa.PrimaryKeyConstraint("id"),
    )
    op.create_index(op.f("ix_tasks_created_at"), "tasks", ["created_at"], unique=False)
    op.create_table(
        "users",
        sa.Column("id", UUIDType(binary=False), nullable=False),
        sa.Column("created_at", sa.DateTime(timezone=True), nullable=False),
        sa.Column("updated_at", sa.DateTime(timezone=True), nullable=False),
        sa.Column("username", sa.String(length=50), nullable=False),
        sa.Column("password", sa.String(length=255), server_default="", nullable=False),
        sa.Column(
            "reset_password_token",
            sa.String(length=100),
            server_default="",
            nullable=False,
        ),
        sa.Column("email", sa.String(length=255), nullable=False),
        sa.Column("email_confirmed_at", sa.DateTime(timezone=True), nullable=True),
        sa.Column(
            "enable_notifications",
            sa.Boolean(),
            server_default=sa.text("true"),
            nullable=False,
        ),
        sa.Column("filters_notifications", JSONType(), nullable=True),
        sa.Column(
            "frequency_notifications",
            sa.Enum("once", "always", name="notification_frequencies"),
            nullable=True,
        ),
        sa.Column(
            "is_active", sa.Boolean(), server_default=sa.text("false"), nullable=False
        ),
        sa.Column(
            "first_name", sa.String(length=100), server_default="", nullable=False
        ),
        sa.Column(
            "last_name", sa.String(length=100), server_default="", nullable=False
        ),
        sa.Column(
            "admin", sa.Boolean(), server_default=sa.text("false"), nullable=True
        ),
        sa.PrimaryKeyConstraint("id"),
        sa.UniqueConstraint("email"),
        sa.UniqueConstraint("username"),
    )
    op.create_index(op.f("ix_users_created_at"), "users", ["created_at"], unique=False)
    op.create_table(
        "vendors",
        sa.Column("id", UUIDType(binary=False), nullable=False),
        sa.Column("created_at", sa.DateTime(timezone=True), nullable=False),
        sa.Column("updated_at", sa.DateTime(timezone=True), nullable=False),
        sa.Column("name", sa.String(), nullable=False),
        sa.PrimaryKeyConstraint("id"),
        sa.UniqueConstraint("name"),
    )
    op.create_index(
        op.f("ix_vendors_created_at"), "vendors", ["created_at"], unique=False
    )
    op.create_table(
        "changes",
        sa.Column("id", UUIDType(binary=False), nullable=False),
        sa.Column("created_at", sa.DateTime(timezone=True), nullable=False),
        sa.Column("updated_at", sa.DateTime(timezone=True), nullable=False),
        sa.Column("json", JSONType(), nullable=True),
        sa.Column("cve_id", UUIDType(binary=False), nullable=True),
        sa.Column("task_id", UUIDType(binary=False), nullable=True),
        sa.ForeignKeyConstraint(
            ["cve_id"],
            ["cves.id"],
        ),
        sa.ForeignKeyConstraint(
            ["task_id"],
            ["tasks.id"],
        ),
        sa.PrimaryKeyConstraint("id"),
    )
    op.create_index(
        op.f("ix_changes_created_at"), "changes", ["created_at"], unique=False
    )
    op.create_table(
        "products",
        sa.Column("id", UUIDType(binary=False), nullable=False),
        sa.Column("created_at", sa.DateTime(timezone=True), nullable=False),
        sa.Column("updated_at", sa.DateTime(timezone=True), nullable=False),
        sa.Column("name", sa.String(), nullable=False),
        sa.Column("vendor_id", UUIDType(binary=False), nullable=True),
        sa.ForeignKeyConstraint(
            ["vendor_id"],
            ["vendors.id"],
        ),
        sa.PrimaryKeyConstraint("id"),
    )
    op.create_index(
        op.f("ix_products_created_at"), "products", ["created_at"], unique=False
    )
    op.create_index(op.f("ix_products_name"), "products", ["name"], unique=False)
    op.create_table(
        "reports",
        sa.Column("id", UUIDType(binary=False), nullable=False),
        sa.Column("created_at", sa.DateTime(timezone=True), nullable=False),
        sa.Column("updated_at", sa.DateTime(timezone=True), nullable=False),
        sa.Column("public_link", sa.String(), nullable=True),
        sa.Column("seen", sa.Boolean(), nullable=True),
        sa.Column("details", JSONType(), nullable=True),
        sa.Column("user_id", UUIDType(binary=False), nullable=True),
        sa.ForeignKeyConstraint(
            ["user_id"],
            ["users.id"],
        ),
        sa.PrimaryKeyConstraint("id"),
    )
    op.create_index(
        op.f("ix_reports_created_at"), "reports", ["created_at"], unique=False
    )
    op.create_table(
        "users_vendors",
        sa.Column("user_id", UUIDType(binary=False), nullable=False),
        sa.Column("vendor_id", UUIDType(binary=False), nullable=False),
        sa.ForeignKeyConstraint(
            ["user_id"],
            ["users.id"],
        ),
        sa.ForeignKeyConstraint(
            ["vendor_id"],
            ["vendors.id"],
        ),
        sa.PrimaryKeyConstraint("user_id", "vendor_id"),
    )
    op.create_table(
        "alerts",
        sa.Column("id", UUIDType(binary=False), nullable=False),
        sa.Column("created_at", sa.DateTime(timezone=True), nullable=False),
        sa.Column("updated_at", sa.DateTime(timezone=True), nullable=False),
        sa.Column("details", JSONType(), nullable=True),
        sa.Column("notify", sa.Boolean(), nullable=True),
        sa.Column("user_id", UUIDType(binary=False), nullable=True),
        sa.Column("cve_id", UUIDType(binary=False), nullable=True),
        sa.Column("report_id", UUIDType(binary=False), nullable=True),
        sa.ForeignKeyConstraint(
            ["cve_id"],
            ["cves.id"],
        ),
        sa.ForeignKeyConstraint(
            ["report_id"],
            ["reports.id"],
        ),
        sa.ForeignKeyConstraint(
            ["user_id"],
            ["users.id"],
        ),
        sa.PrimaryKeyConstraint("id"),
    )
    op.create_index(
        op.f("ix_alerts_created_at"), "alerts", ["created_at"], unique=False
    )
    op.create_table(
        "events",
        sa.Column("id", UUIDType(binary=False), nullable=False),
        sa.Column("created_at", sa.DateTime(timezone=True), nullable=False),
        sa.Column("updated_at", sa.DateTime(timezone=True), nullable=False),
        sa.Column(
            "type",
            sa.Enum(
                "new_cve",
                "references",
                "cpes",
                "cvss",
                "summary",
                "cwes",
                name="event_types",
            ),
            nullable=True,
        ),
        sa.Column("details", JSONType(), nullable=True),
        sa.Column("review", sa.Boolean(), nullable=True),
        sa.Column("cve_id", UUIDType(binary=False), nullable=True),
        sa.Column("change_id", UUIDType(binary=False), nullable=True),
        sa.ForeignKeyConstraint(
            ["change_id"],
            ["changes.id"],
        ),
        sa.ForeignKeyConstraint(
            ["cve_id"],
            ["cves.id"],
        ),
        sa.PrimaryKeyConstraint("id"),
    )
    op.create_index(
        op.f("ix_events_created_at"), "events", ["created_at"], unique=False
    )
    op.create_table(
        "users_products",
        sa.Column("user_id", UUIDType(binary=False), nullable=False),
        sa.Column("product_id", UUIDType(binary=False), nullable=False),
        sa.ForeignKeyConstraint(
            ["product_id"],
            ["products.id"],
        ),
        sa.ForeignKeyConstraint(
            ["user_id"],
            ["users.id"],
        ),
        sa.PrimaryKeyConstraint("user_id", "product_id"),
    )
    op.create_table(
        "alerts_events",
        sa.Column("alert_id", UUIDType(binary=False), nullable=False),
        sa.Column("event_id", UUIDType(binary=False), nullable=False),
        sa.ForeignKeyConstraint(
            ["alert_id"],
            ["alerts.id"],
        ),
        sa.ForeignKeyConstraint(
            ["event_id"],
            ["events.id"],
        ),
        sa.PrimaryKeyConstraint("alert_id", "event_id"),
    )


def downgrade():
    op.drop_table("alerts_events")
    op.drop_table("users_products")
    op.drop_index(op.f("ix_events_created_at"), table_name="events")
    op.drop_table("events")
    op.execute("DROP TYPE event_types")
    op.drop_index(op.f("ix_alerts_created_at"), table_name="alerts")
    op.drop_table("alerts")
    op.drop_table("users_vendors")
    op.drop_index(op.f("ix_reports_created_at"), table_name="reports")
    op.drop_table("reports")
    op.drop_index(op.f("ix_products_name"), table_name="products")
    op.drop_index(op.f("ix_products_created_at"), table_name="products")
    op.drop_table("products")
    op.drop_index(op.f("ix_changes_created_at"), table_name="changes")
    op.drop_table("changes")
    op.drop_index(op.f("ix_vendors_created_at"), table_name="vendors")
    op.drop_table("vendors")
    op.drop_index(op.f("ix_users_created_at"), table_name="users")
    op.drop_table("users")
    op.execute("DROP TYPE notification_frequencies")
    op.drop_index(op.f("ix_tasks_created_at"), table_name="tasks")
    op.drop_table("tasks")
    op.drop_index(op.f("ix_metas_created_at"), table_name="metas")
    op.drop_table("metas")
    op.drop_index(op.f("ix_cwes_created_at"), table_name="cwes")
    op.drop_table("cwes")
    op.drop_index(op.f("ix_cves_updated_at"), table_name="cves")
    op.drop_index(op.f("ix_cves_cve_id"), table_name="cves")
    op.drop_index(op.f("ix_cves_created_at"), table_name="cves")
    op.drop_index(op.f("ix_cves_vendors"), table_name="cves")
    op.drop_index(op.f("ix_cves_cwes"), table_name="cves")
    op.drop_table("cves")



from flask import request
from flask_restful import fields, marshal_with

from opencve.api.base import BaseResource
from opencve.api.cves import cves_fields
from opencve.api.fields import HumanizedNameField
from opencve.controllers.cves import CveController
from opencve.controllers.products import ProductController
from opencve.controllers.vendors import VendorController


product_fields = {
    "name": fields.String(attribute="name"),
    "human_name": HumanizedNameField(attribute="name"),
}

vendor_product_fields = {
    **product_fields,
    "vendor_name": fields.String(attribute="vendor.name"),
    "vendor_human_name": HumanizedNameField(attribute="vendor.name"),
}


class ProductListResource(BaseResource):
    @marshal_with(product_fields)
    def get(self, vendor):
        return ProductController.list_items(
            {
                **request.args,
                "vendor": vendor,
                "product_page": request.args.get("page", 1),
            }
        )


class ProductResource(BaseResource):
    @marshal_with(product_fields)
    def get(self, vendor, product):
        return ProductController.get({"vendor": vendor, "product": product})


class ProductCveResource(BaseResource):
    @marshal_with(cves_fields)
    def get(self, vendor, product):
        ProductController.get({"vendor": vendor, "product": product})
        return CveController.list_items(
            {**request.args, "vendor": vendor, "product": product}
        )


class FlatProductListResource(BaseResource):
    @marshal_with(vendor_product_fields)
    def get(self):
        return ProductController.list_items(
            {**request.args, "product_page": request.args.get("page", 1)}
        )



import pytest

from flask import g

from opencve import create_app
from opencve.extensions import db


@pytest.fixture
def app():
    _app = create_app(
        "test",
        {
            "RATELIMIT_ENABLED": True,
            "RATELIMIT_VALUE": "2/minute",
            "RATELIMIT_STORAGE_URL": "memory://",
        },
    )
    ctx = _app.app_context()
    ctx.push()
    db.create_all()
    yield _app


def test_ratelimit(app, client, create_user, create_vendor):
    create_user("opencve")
    client.login("opencve")

    def _do_request(endpoint="/cve"):
        resp = client.get("/api{}".format(endpoint))

        # g._rate_limiting_complete must be set to False after each request
        # see https://github.com/alisaifee/flask-limiter/issues/147#issuecomment-642683820
        g._rate_limiting_complete = False

        return resp

    # Rate limit is disabled
    assert _do_request().status_code == 200
    assert _do_request().status_code == 200
    assert _do_request().status_code == 429

    # Ratelimit is shared between all routes
    assert _do_request("/reports").status_code == 429
    assert _do_request("/vendors").status_code == 429
    assert _do_request("/cwe").status_code == 429

    # Other user is not rate limited
    create_user("john")
    client.login("john")
    assert _do_request().status_code == 200



def test_list_cwes_authentication(client, create_user):
    create_user("opencve")
    response = client.get("/api/cwe")
    assert response.status_code == 401
    response = client.login("john").get("/api/cwe")
    assert response.status_code == 401
    response = client.login("opencve").get("/api/cwe")
    assert response.status_code == 200


def test_list_cwes(client, create_user, create_cwe):
    create_user("opencve")
    response = client.login("opencve").get("/api/cwe")
    assert response.status_code == 200
    assert response.json == []

    create_cwe("CWE-123", "The name", "The description")
    response = client.login("opencve").get("/api/cwe")
    assert response.status_code == 200
    assert len(response.json) == 1
    assert response.json[0] == {
        "id": "CWE-123",
        "name": "The name",
        "description": "The description",
    }


def test_get_cwe_not_found(client, create_user):
    create_user("opencve")
    response = client.login("opencve").get("/api/cwe/404")
    assert response.status_code == 404
    assert response.json == {"message": "Not found."}


def test_get_cwe(client, create_user, create_cwe):
    create_user("opencve")
    create_cwe("CWE-123", "The name", "The description")

    response = client.login("opencve").get("/api/cwe/CWE-123")
    assert response.status_code == 200
    assert response.json == {
        "id": "CWE-123",
        "name": "The name",
        "description": "The description",
    }


def test_list_cwe_cves(client, create_user, create_cve):
    create_user("opencve")
    create_cve("CVE-2018-18074")

    response = client.login("opencve").get("/api/cwe/CWE-522/cve")
    assert len(response.json) == 1

    assert response.json[0] == {
        "created_at": "2018-10-09T17:29:00Z",
        "id": "CVE-2018-18074",
        "summary": "The Requests package before 2.20.0 for Python sends an HTTP Authorization header to an http URI upon receiving a same-hostname https-to-http redirect, which makes it easier for remote attackers to discover credentials by sniffing the network.",
        "updated_at": "2019-10-03T00:03:00Z",
    }



def test_list_no_cwes(client):
    response = client.get("/cwe")
    assert response.status_code == 200
    assert b"No CWE found." in response.data


def taest_list_cwes(client, create_cwes, create_cwe):
    create_cwes("CWE-1", "Name of the fist CWE", "Description of the first CWE")
    create_cwes("CWE-2", "Name of the second CWE", "Description of the second CWE")

    response = client.get("/cwe")
    assert response.status_code == 200
    assert b"CWE-1" in response.data
    assert b"Name of the fist CWE" in response.data
    assert b"Description of the first CWE" in response.data
    assert b"CWE-2" in response.data
    assert b"Name of the second CWE" in response.data
    assert b"Description of the second CWE" in response.data



import importlib
import pkgutil
from pathlib import Path

from flask import Flask, jsonify, render_template, request

from opencve.api import api_bp
from opencve.context import _is_active
from opencve.controllers.main import main, welcome
from opencve.extensions import db
from opencve.settings import env_config


with open(Path(__file__).parent.resolve() / "VERSION", encoding="utf-8") as version:
    __version__ = version.readline().rstrip()


def not_found(e):
    """Handles 404 errors."""
    if request.path.startswith("/api"):
        return jsonify({"error": "Not found."}), 404
    return render_template("errors/404.html"), 404


def create_app(environment="production", custom_config={}):
    app = Flask(__name__)
    app.config.from_object(env_config[environment])

    # Customize configuration
    for conf, value in custom_config.items():
        app.config[conf] = value

    # Load extensions
    env_config[environment].init_app(app)
    app.extensions["mail"].debug = 0

    # Register the blueprints
    app.register_blueprint(api_bp, url_prefix="/api")
    app.register_blueprint(main)
    app.register_blueprint(welcome)

    # Error handlers
    app.register_error_handler(404, not_found)

    @app.context_processor
    def is_active():
        return {"is_active": _is_active}

    return app


def import_submodules(package, modules_to_import):
    """Import all submodules of a module."""
    if isinstance(package, str):
        package = importlib.import_module(package)
    results = {}

    for loader, name, is_pkg in pkgutil.walk_packages(package.__path__):
        if not name.startswith("_"):
            full_name = package.__name__ + "." + name

            if any((x in package.__name__ for x in modules_to_import)):
                results[full_name] = importlib.import_module(full_name)

            elif any((x in name for x in modules_to_import)):
                results[full_name] = importlib.import_module(full_name)

            if is_pkg and name in modules_to_import:
                results.update(import_submodules(full_name, modules_to_import))
    return results


import_submodules(
    __name__,
    (
        "api",
        "checks",
        "controllers",
        "models",
        "tasks",
        "views",
    ),
)



from flask import current_app as app
from flask import request, url_for
from flask_user import current_user

from opencve.constants import EVENT_TYPES, PRODUCT_SEPARATOR
from opencve.models.tags import UserTag


def _cvss_percent(score):
    percent = score * 100 / 10
    return "{}%".format(str(percent))


def _cvss_bg(score):
    score = float(score)

    if 0 <= score <= 3.9:
        return ("bg-blue", "label-info")
    elif 4.0 <= score <= 6.9:
        return ("bg-yellow", "label-warning")
    elif 7.0 <= score <= 8.9:
        return ("bg-red", "label-danger")
    else:
        return ("bg-critical", "label-critical")


def _metric_bg(version, type, value):
    metrics_v2 = {
        "AV": {
            "local": "label-default",
            "adjacent network": "label-warning",
            "network": "label-danger",
        },
        "AC": {
            "high": "label-default",
            "medium": "label-warning",
            "low": "label-danger",
        },
        "AU": {
            "multiple": "label-default",
            "single": "label-warning",
            "none": "label-danger",
        },
        "C": {
            "none": "label-default",
            "partial": "label-warning",
            "complete": "label-danger",
        },
        "I": {
            "none": "label-default",
            "partial": "label-warning",
            "complete": "label-danger",
        },
        "A": {
            "none": "label-default",
            "partial": "label-warning",
            "complete": "label-danger",
        },
    }

    metrics_v3 = {
        "AV": {
            "network": "label-danger",
            "adjacent": "label-warning",
            "local": "label-warning",
            "physical": "label-default",
        },
        "AC": {"low": "label-danger", "high": "label-warning"},
        "PR": {"none": "label-danger", "low": "label-warning", "high": "label-default"},
        "UI": {"none": "label-danger", "required": "label-warning"},
        "S": {"unchanged": "label-default", "changed": "label-danger"},
        "C": {"high": "label-danger", "low": "label-warning", "none": "label-default"},
        "I": {"high": "label-danger", "low": "label-warning", "none": "label-default"},
        "A": {"high": "label-danger", "low": "label-warning", "none": "label-default"},
    }
    versions = {"v2": metrics_v2, "v3": metrics_v3}

    try:
        value = versions[version][type][value.lower()]
    except KeyError:
        return ("label-default", "No description")

    return value


def _humanize_filter(s):
    return " ".join(map(lambda x: x.capitalize(), s.split("_")))


def _excerpt(objects, _type):
    """
    This function takes a flat list of vendors and products and returns
    the HTML code used in the CVEs list page.
    """
    output = ""

    if not objects:
        return output

    # Keep the objects of the requested type
    if _type == "products":
        objects = [o for o in objects if PRODUCT_SEPARATOR in o]
    else:
        objects = [o for o in objects if not PRODUCT_SEPARATOR in o]

    objects = sorted(objects)
    output += '<span class="badge badge-primary">{}</span> '.format(len(objects))

    # Keep the remains size and reduce the list
    remains = len(objects[app.config["COUNT_EXCERPT"] :])

    if len(objects) > app.config["COUNT_EXCERPT"]:
        objects = objects[: app.config["COUNT_EXCERPT"]]

    # Construct the HTML
    for idx, obj in enumerate(objects):
        if _type == "products":
            vendor, product = obj.split(PRODUCT_SEPARATOR)
            url = url_for("main.cves", vendor=vendor, product=product)
            output += f"<a href='{url}'>{_humanize_filter(product)}</a>"
        elif _type == "vendors":
            url = url_for("main.cves", vendor=obj)
            output += f"<a href='{url}'>{_humanize_filter(obj)}</a>"
        else:
            url = url_for("main.cves", tag=obj)
            tag = UserTag.query.filter_by(user_id=current_user.id, name=obj).first()
            output += f"<a href='{url}'><span class='label label-tag' style='background-color: {tag.color};'>{obj}</span></a>"

        output += ", " if idx + 1 != len(objects) and _type != "tags" else " "

    if remains:
        output += "<i>and {} more</i>".format(remains)

    return output


def _report_excerpt(items):
    output = ""

    if not items:
        return output

    items_copy = items
    if len(items) > app.config["REPORT_COUNT_EXCERPT"]:
        items = items[: app.config["REPORT_COUNT_EXCERPT"]]

    for idx, item in enumerate(items):
        output += _humanize_filter(item)
        if idx + 1 != len(items):
            output += ", "
        else:
            output += " "

    remains = len(items_copy[app.config["REPORT_COUNT_EXCERPT"] :])

    if remains:
        output += "<i>and {} more</i>".format(remains)

    return output


def _is_active(route):
    return request.endpoint in route.split(",")


def _event_excerpt(details):
    if isinstance(details, list):
        return f"<strong>{len(details)}</strong> added"
    else:
        output = []
        if "changed" in details:
            output.append(f"<strong>{len(details['changed'])}</strong> changed")
        if "added" in details:
            output.append(f"<strong>{len(details['added'])}</strong> added")
        if "removed" in details:
            output.append(f"<strong>{len(details['removed'])}</strong> removed")
        return ", ".join(output)


def _event_description(code):
    return dict(EVENT_TYPES)[code]


def _remove_product_separator(s):
    return s.replace(PRODUCT_SEPARATOR, " ")



import pytest
from werkzeug.exceptions import NotFound

from opencve.controllers.products import ProductController
from opencve.models.vendors import Vendor


def test_metas(app, create_vendor):
    create_vendor("vendor1", "product1")

    with app.test_request_context():
        _, metas, _ = ProductController.list({"vendor": "vendor1"})
    assert metas == {}


def test_list_vendor_notfound(app):
    with app.test_request_context():
        with pytest.raises(NotFound):
            ProductController.list({"vendor": "notfound"})


def test_list(app, create_vendor):
    create_vendor("vendor1", "product1")
    create_vendor("vendor1", "product2")

    with app.test_request_context():
        products = ProductController.list_items({"vendor": "vendor1"})
    assert len(products) == 2
    assert sorted([p.name for p in products]) == ["product1", "product2"]


def test_list_paginated(app, create_vendor):
    old = app.config["PRODUCTS_PER_PAGE"]
    app.config["PRODUCTS_PER_PAGE"] = 3

    create_vendor("vendor1", "product1")
    create_vendor("vendor1", "product2")
    create_vendor("vendor1", "product3")
    create_vendor("vendor1", "product4")

    with app.test_request_context():
        products = ProductController.list_items({"vendor": "vendor1"})
        assert sorted([p.name for p in products]) == [
            "product1",
            "product2",
            "product3",
        ]
        products = ProductController.list_items(
            {"vendor": "vendor1", "product_page": 2}
        )
        assert sorted([p.name for p in products]) == ["product4"]

    app.config["PRODUCTS_PER_PAGE"] = old


@pytest.mark.parametrize(
    "args,result",
    [
        ({"search": "nonexistingkeyword"}, []),
        ({"search": "1"}, ["product_1"]),
        ({"search": "product"}, ["product_1", "product_2"]),
        ({"search": "product 2"}, ["product_2"]),
    ],
)
def test_by_search(app, create_vendor, args, result):
    create_vendor("vendor1", "product_1")
    create_vendor("vendor1", "product_2")

    with app.test_request_context():
        products = ProductController.list_items({"vendor": "vendor1", **args})
    assert sorted([p.name for p in products]) == result



from sqlalchemy_utils import ChoiceType, JSONType, UUIDType

from opencve.constants import EVENT_TYPES
from opencve.extensions import db
from opencve.models import BaseModel, alerts_events


class Event(BaseModel):
    __tablename__ = "events"

    type = db.Column(ChoiceType(EVENT_TYPES))
    details = db.Column(JSONType)
    review = db.Column(db.Boolean, default=False)

    # Relationships
    cve_id = db.Column(UUIDType(binary=False), db.ForeignKey("cves.id"))
    cve = db.relationship("Cve", back_populates="events")
    change_id = db.Column(UUIDType(binary=False), db.ForeignKey("changes.id"))
    change = db.relationship("Change", back_populates="events")
    alerts = db.relationship("Alert", secondary=alerts_events, passive_deletes=True)

    def __repr__(self):
        return "<Event {}>".format(self.type)



import pytest
from werkzeug.exceptions import NotFound

from opencve.controllers.vendors import VendorController


def test_metas(app):
    with app.test_request_context():
        _, metas, _ = VendorController.list()
    assert metas == {}


def test_list(app, create_vendor):
    create_vendor("vendor1", "product1")
    create_vendor("vendor2", "product2")

    with app.test_request_context():
        vendors = VendorController.list_items()
    assert len(vendors) == 2
    assert sorted([v.name for v in vendors]) == ["vendor1", "vendor2"]


def test_list_paginated(app, create_vendor):
    old = app.config["VENDORS_PER_PAGE"]
    app.config["VENDORS_PER_PAGE"] = 3

    create_vendor("vendor1", "product1")
    create_vendor("vendor2", "product2")
    create_vendor("vendor3", "product3")
    create_vendor("vendor4", "product4")

    with app.test_request_context():
        vendors = VendorController.list_items()
        assert sorted([v.name for v in vendors]) == ["vendor1", "vendor2", "vendor3"]
        vendors = VendorController.list_items({"page": 2})
        assert sorted([v.name for v in vendors]) == ["vendor4"]

    app.config["VENDORS_PER_PAGE"] = old


@pytest.mark.parametrize(
    "args,result",
    [
        ({"search": "nonexistingkeyword"}, []),
        ({"search": "1"}, ["vendor_1"]),
        ({"search": "vendor"}, ["vendor_1", "vendor_2"]),
        ({"search": "vendor 2"}, ["vendor_2"]),
    ],
)
def test_by_search(app, create_vendor, args, result):
    create_vendor("vendor_1", "product1")
    create_vendor("vendor_2", "product2")

    with app.test_request_context():
        vendors = VendorController.list_items(args)
    assert sorted([v.name for v in vendors]) == result



from flask import current_app as app
from flask_login import current_user
from flask_user.forms import unique_email_validator
from flask_wtf import FlaskForm
from wtforms import (
    BooleanField,
    PasswordField,
    RadioField,
    SelectField,
    StringField,
    SubmitField,
    validators,
)

from opencve.constants import CVSS_SCORES, FREQUENCIES_TYPES


class ChangePasswordForm(FlaskForm):
    old_password = PasswordField(
        "Password", validators=[validators.DataRequired("Password is required")]
    )
    new_password = PasswordField(
        "Password", validators=[validators.DataRequired("Password is required")]
    )
    submit = SubmitField("Change password")

    def validate(self):
        user_manager = app.user_manager

        if not super(ChangeEmailForm, self).validate():
            return False

        if not current_user or not user_manager.verify_password(
            self.password.data, current_user
        ):
            self.password.errors.append("Password is incorrect")
            return False

        return True


class ChangeEmailForm(FlaskForm):
    email = StringField(
        "New email",
        validators=[
            validators.DataRequired("Email is required"),
            validators.Email("Invalid email"),
            unique_email_validator,
        ],
    )
    password = PasswordField(
        "Password", validators=[validators.DataRequired("Password is required")]
    )
    submit = SubmitField("Change email")

    def validate(self):
        user_manager = app.user_manager

        if not super(ChangeEmailForm, self).validate():
            return False

        if not current_user or not user_manager.verify_password(
            self.password.data, current_user
        ):
            self.password.errors.append("Password is incorrect")
            return False

        return True


class MailNotificationsForm(FlaskForm):
    enable = RadioField(
        "Enable email notifications", choices=[("yes", "Yes"), ("no", "No")]
    )
    frequency = SelectField("Email frequency", choices=FREQUENCIES_TYPES)
    submit = SubmitField("Save changes")


class FiltersNotificationForm(FlaskForm):
    new_cve = BooleanField("New CVE")
    first_time = BooleanField("Subscription appeared for the first time")
    references = BooleanField("Reference changed")
    cvss = BooleanField("CVSS changed")
    cpes = BooleanField("CPE changed")
    summary = BooleanField("Summary changed")
    cwes = BooleanField("CWE changed")
    cvss_score = SelectField("CVSS score", coerce=int, choices=CVSS_SCORES)
    submit = SubmitField("Save changes")


class TagForm(FlaskForm):
    name = StringField(
        "Name",
        validators=[
            validators.DataRequired("Name is required"),
            validators.Regexp(
                "^[a-zA-Z0-9\-_]+$",
                message="Only alphanumeric, dash and underscore characters are accepted",
            ),
        ],
    )
    description = StringField("Description")
    color = StringField(
        "Color",
        validators=[
            validators.DataRequired("Color is required"),
            validators.Regexp(
                "^#[0-9a-fA-F]{6}$", message="Color must be in hexadecimal format"
            ),
        ],
        default="#000000",
    )
    submit = SubmitField("Save")


class ActivitiesViewForm(FlaskForm):
    view = RadioField(
        "Activities View",
        choices=[
            ("all", "Display all activities"),
            ("subscriptions", "Display subscriptions activities"),
        ],
    )



import datetime

from opencve.commands.utils import CveUtil
from opencve.constants import PRODUCT_SEPARATOR
from opencve.extensions import db
from opencve.models.changes import Change
from opencve.models.cve import Cve
from opencve.models.cwe import Cwe
from opencve.models.events import Event
from opencve.models.tasks import Task
from opencve.models.vendors import Vendor
from opencve.models.products import Product


def test_create_cve(app, open_file):
    cve = CveUtil.create_cve(open_file("cves/CVE-2020-26116.json"))

    cves = Cve.query.all()
    assert len(cves) == 1

    # The CVE has been created
    assert cve.id == cves[0].id
    assert cve.cve_id == "CVE-2020-26116"
    assert cve.cwes == ["CWE-116"]
    assert sorted(cve.vendors) == sorted(
        [
            "fedoraproject",
            f"fedoraproject{PRODUCT_SEPARATOR}fedora",
            "python",
            f"python{PRODUCT_SEPARATOR}python",
        ]
    )
    assert (
        cve.summary
        == "http.client in Python 3.x before 3.5.10, 3.6.x before 3.6.12, 3.7.x before 3.7.9, and 3.8.x before 3.8.5 allows CRLF injection if the attacker controls the HTTP request method, as demonstrated by inserting CR and LF control characters in the first argument of HTTPConnection.request."
    )
    assert cve.cvss2 == 6.4
    assert cve.cvss3 == 7.2
    assert cve.events == []
    assert cve.changes == []
    assert cve.alerts == []
    assert round(cve.cvss_weight, 1) == 13.6

    # The CWE has been created
    cwes = Cwe.query.all()
    assert len(cwes) == 1
    cwe = cwes[0]
    assert cwe.cwe_id == "CWE-116"

    # The vendors and products has been created
    vendors = Vendor.query.all()
    assert len(vendors) == 2

    vendor_1 = Vendor.query.filter_by(name="fedoraproject").first()
    assert len(vendor_1.products) == 1
    assert vendor_1.products[0].name == "fedora"

    vendor_2 = Vendor.query.filter_by(name="python").first()
    assert len(vendor_2.products) == 1
    assert vendor_2.products[0].name == "python"


def test_has_changed(app, open_file):
    cve_json = open_file("cves/CVE-2020-26116.json")
    cve_db = CveUtil.create_cve(cve_json)

    cve_db.updated_at = datetime.datetime.now() - datetime.timedelta(days=1)
    db.session.commit()

    assert CveUtil.cve_has_changed(cve_db, cve_json)


def test_create_event(app, open_file):
    cve_json = open_file("cves/CVE-2020-26116.json")
    cve = CveUtil.create_cve(cve_json)

    event = CveUtil.create_event(cve, cve_json, "new_cve", {"foo": "bar"})
    assert Event.query.first().id == event.id

    assert event.type == "new_cve"
    assert event.details == {"foo": "bar"}
    assert event.review == False
    assert event.cve_id == cve.id
    assert event.cve.id == cve.id
    assert event.change == None
    assert event.change_id == None
    assert event.alerts == []


def test_create_change(open_file):
    task = Task()
    db.session.add(task)
    db.session.commit()

    cve_json = open_file("cves/CVE-2020-26116.json")
    cve = CveUtil.create_cve(cve_json)

    change = CveUtil.create_change(cve, cve_json, task, [])
    assert Change.query.first().id == change.id

    assert change.json == cve_json
    assert change.cve_id == cve.id
    assert change.cve.id == cve.id
    assert change.task_id == task.id
    assert change.task.id == task.id
    assert change.events == []



import arrow
from nested_lookup import nested_lookup

from opencve.commands import info
from opencve.extensions import db
from opencve.models.changes import Change
from opencve.models.cve import Cve
from opencve.models.cwe import Cwe
from opencve.models.events import Event
from opencve.models.products import Product
from opencve.models.vendors import Vendor
from opencve.utils import convert_cpes, flatten_vendors, get_cwes


class CveUtil(object):
    @classmethod
    def cve_has_changed(cls, cve_db, cve_json):
        return arrow.get(cve_json["lastModifiedDate"]) != cve_db.updated_at

    @classmethod
    def create_event(cls, cve_obj, cve_json, type, payload={}):
        event = Event(
            created_at=arrow.get(cve_json["lastModifiedDate"]).datetime,
            updated_at=arrow.get(cve_json["lastModifiedDate"]).datetime,
            cve=cve_obj,
            type=type,
            details=payload,
            review=False,
        )
        db.session.add(event)
        db.session.commit()

        info("Event {} created (ID: {})".format(type, event.id))
        return event

    @classmethod
    def create_change(cls, cve_obj, cve_json, task, events):
        change = Change(
            created_at=arrow.get(cve_json["lastModifiedDate"]).datetime,
            updated_at=arrow.get(cve_json["lastModifiedDate"]).datetime,
            cve=cve_obj,
            task=task,
            events=events,
            json=cve_json,
        )
        db.session.add(change)
        db.session.commit()

        info("Change created (ID: {})".format(change.id))
        return change

    @classmethod
    def create_cve(cls, cve_json):
        cvss2 = (
            cve_json["impact"]["baseMetricV2"]["cvssV2"]["baseScore"]
            if "baseMetricV2" in cve_json["impact"]
            else None
        )
        cvss3 = (
            cve_json["impact"]["baseMetricV3"]["cvssV3"]["baseScore"]
            if "baseMetricV3" in cve_json["impact"]
            else None
        )

        # Construct CWE and CPE lists
        cwes = get_cwes(
            cve_json["cve"]["problemtype"]["problemtype_data"][0]["description"]
        )
        cpes = convert_cpes(cve_json["configurations"])
        vendors = flatten_vendors(cpes)

        # Create the CVE
        cve = Cve(
            cve_id=cve_json["cve"]["CVE_data_meta"]["ID"],
            summary=cve_json["cve"]["description"]["description_data"][0]["value"],
            json=cve_json,
            vendors=vendors,
            cwes=cwes,
            cvss2=cvss2,
            cvss3=cvss3,
            created_at=arrow.get(cve_json["publishedDate"]).datetime,
            updated_at=arrow.get(cve_json["lastModifiedDate"]).datetime,
        )
        db.session.add(cve)
        db.session.commit()

        # Add the CWE that not exists yet in database
        for cwe in cwes:
            cwe_obj = Cwe.query.filter_by(cwe_id=cwe).first()
            if not cwe_obj:
                info(
                    f"{cwe} detected in {cve.cve_id} but not existing in database, adding it..."
                )
                cwe_obj = Cwe(cwe_id=cwe)
                db.session.add(cwe_obj)
                db.session.commit()

        # Add the CPEs
        vendors_products = convert_cpes(
            nested_lookup("cpe23Uri", cve_json["configurations"])
        )
        for vendor, products in vendors_products.items():
            v_obj = Vendor.query.filter_by(name=vendor).first()

            # Create the vendor
            if not v_obj:
                v_obj = Vendor(name=vendor)
                db.session.add(v_obj)
                db.session.commit()

            # Create the products
            for product in products:
                p_obj = Product.query.filter_by(name=product, vendor=v_obj).first()
                if not p_obj:
                    p_obj = Product(name=product, vendor=v_obj)
                    db.session.add(p_obj)
                    db.session.commit()

        return cve



import time
from contextlib import contextmanager
from functools import update_wrapper
from pathlib import Path

import click


def info(msg, nl=True):
    click.echo("[*] {}".format(msg), nl=nl)


def error(msg, nl=True):
    click.echo("[error] {}".format(msg), nl=nl)


def header(msg):
    click.echo("#" * len(msg))
    click.echo(msg)
    click.echo("#" * len(msg))


@contextmanager
def timed_operation(msg, nl=False):
    start = time.time()
    info(msg, nl=nl)
    yield
    click.echo(" (done in {}s).".format(round(time.time() - start, 3)))


def ensure_config(f):
    @click.pass_context
    def decorator(__ctx, *args, **kwargs):
        from opencve.configuration import OPENCVE_CONFIG

        if not Path(OPENCVE_CONFIG).exists():
            error("Configuration not found (use the 'init' command)")
            __ctx.exit()
        return __ctx.invoke(f, *args, **kwargs)

    return update_wrapper(decorator, f)



from flask import abort, current_app as app
from flask_paginate import Pagination
from werkzeug.datastructures import ImmutableMultiDict


class BaseController(object):
    model = None
    order = None
    per_page_param = None
    page_parameter = "page"
    schema = {}

    @classmethod
    def build_query(cls, args):
        return cls.model.query, {}

    @classmethod
    def parse_args(cls, args):
        args = ImmutableMultiDict(args)

        parsed_args = {
            cls.page_parameter: args.get(cls.page_parameter, type=int, default=1)
        }

        for key in args.keys():
            if key in cls.schema.keys():
                parsed_args[key] = args.get(
                    key,
                    type=cls.schema.get(key).get("type"),
                    default=cls.schema.get(key).get("default"),
                )
        return parsed_args

    @classmethod
    def get(cls, filters):
        obj = cls.model.query.filter_by(**filters).first()
        if not obj:
            abort(404, "Not found.")
        return obj

    @classmethod
    def list(cls, args={}):
        args = cls.parse_args(args)
        query, metas = cls.build_query(args)

        objects = query.order_by(*cls.order).paginate(
            args.get(cls.page_parameter), app.config[cls.per_page_param], True
        )

        pagination = cls.get_pagination(args, objects)
        return objects, metas, pagination

    @classmethod
    def get_pagination(cls, args, objects):
        return Pagination(
            page=args.get(cls.page_parameter),
            total=objects.total,
            per_page=app.config[cls.per_page_param],
            page_parameter=cls.page_parameter,
            record_name="objects",
            css_framework="bootstrap3",
        )

    @classmethod
    def list_items(cls, args={}):
        objects, _, _ = cls.list(args)
        return objects.items



from datetime import datetime, timezone
from unittest.mock import patch

from flask import request

from opencve.extensions import db
from opencve.models.reports import Report
from opencve.models.users import User
from opencve.models.vendors import Vendor
from opencve.tasks.alerts import handle_alerts
from opencve.tasks.reports import handle_reports


def test_redirect_auth(client):
    response = client.get("/reports")
    assert response.status_code == 302

    with client:
        response = client.get("/reports", follow_redirects=True)
        assert response.status_code == 200
        assert request.path == "/login"


def test_no_reports(client, login):
    response = client.get("/reports")
    assert response.status_code == 200
    assert b"No report yet." in response.data


@patch("opencve.tasks.reports.user_manager.email_manager.send_user_report")
def test_list_reports(mock_send, client, login, handle_events):
    user = User.query.first()
    handle_events("modified_cves/CVE-2018-18074.json")
    user.vendors.append(Vendor.query.filter_by(name="canonical").first())
    db.session.commit()
    handle_alerts()
    handle_reports()

    report = Report.query.first()
    report.created_at = datetime(2021, 1, 1, tzinfo=timezone.utc)

    response = client.get("/reports")
    assert response.status_code == 200
    assert b"Canonical" in response.data
    assert b"Jan 01 &#39;21 at 00:00" in response.data
    db.session.commit()


@patch("opencve.tasks.reports.user_manager.email_manager.send_user_report")
def test_get_report(mock_send, client, login, handle_events):
    user = User.query.first()
    handle_events("modified_cves/CVE-2018-18074.json")
    user.vendors.append(Vendor.query.filter_by(name="canonical").first())
    db.session.commit()
    handle_alerts()
    handle_reports()

    report = Report.query.first()
    report.created_at = datetime(2021, 1, 1, tzinfo=timezone.utc)

    response = client.get(f"/reports/{report.public_link}")
    assert b"1 alert on 01/01/21" in response.data
    assert b"Canonical" in response.data
    assert b"CVE-2018-18074" in response.data
    assert b"9.8" in response.data
    assert b"New&nbsp;CVE" in response.data
    db.session.commit()



from sqlalchemy_utils import JSONType, UUIDType

from opencve.extensions import db
from opencve.models import BaseModel


class Change(BaseModel):
    __tablename__ = "changes"

    json = db.Column(JSONType)

    # Relationships
    cve_id = db.Column(UUIDType(binary=False), db.ForeignKey("cves.id"))
    cve = db.relationship("Cve", back_populates="changes")

    task_id = db.Column(UUIDType(binary=False), db.ForeignKey("tasks.id"))
    task = db.relationship("Task", back_populates="changes")

    events = db.relationship("Event", back_populates="change")



import os
import pathlib
import shutil

import click
from flask.cli import with_appcontext

from opencve.configuration import (
    DEFAULT_CONFIG,
    DEFAULT_WELCOME_FILES,
    OPENCVE_CONFIG,
    OPENCVE_HOME,
    OPENCVE_WELCOME_FILES,
)
from opencve.commands import info, error


def create_config():
    if not pathlib.Path(OPENCVE_CONFIG).exists():

        # Do not create the home if user directly specifies the config path
        if not os.environ.get("OPENCVE_CONFIG"):
            pathlib.Path(OPENCVE_HOME).mkdir(parents=True, exist_ok=True)

        with open(DEFAULT_CONFIG) as f:
            conf = f.read()

        # Generate a unique secret key
        conf = conf.replace("{SECRET_KEY}", os.urandom(32).hex())

        with open(OPENCVE_CONFIG, "w") as f:
            f.write(conf)

        # Copy the welcome files
        shutil.copytree(DEFAULT_WELCOME_FILES, OPENCVE_WELCOME_FILES)

        return OPENCVE_CONFIG, True

    return OPENCVE_CONFIG, False


@click.command()
@with_appcontext
def init():
    """Initialize the configuration file."""
    path, created = create_config()
    if created:
        info(f"Configuration created in {path}")
    else:
        error(f"Configuration already exists ({path})")



import random
import string

from sqlalchemy_utils import JSONType, UUIDType

from opencve.extensions import db
from opencve.models import BaseModel


def generate_public_link(size=12):
    chars = string.ascii_uppercase + string.digits
    return "".join(random.choice(chars) for _ in range(size))


class Report(BaseModel):
    __tablename__ = "reports"

    public_link = db.Column(db.String(), default=generate_public_link)
    seen = db.Column(db.Boolean(), default=False)
    details = db.Column(JSONType)

    user_id = db.Column(UUIDType(binary=False), db.ForeignKey("users.id"))
    user = db.relationship("User", back_populates="reports")

    alerts = db.relationship(
        "Alert",
        back_populates="report",
        cascade="all, delete",
        passive_deletes=True,
    )

    def __repr__(self):
        return "<Report {}>".format(self.id)



from opencve.constants import PRODUCT_SEPARATOR
from opencve.context import _excerpt


def test_vendors_excerpt_without_more():
    result = _excerpt(
        [
            "vendor1",
            "vendor2",
            f"vendor1{PRODUCT_SEPARATOR}product1",
            f"vendor2{PRODUCT_SEPARATOR}product2",
        ],
        "vendors",
    )

    assert '<span class="badge badge-primary">2</span>' in result
    assert "Vendor1" in result
    assert "cve?vendor=vendor1" in result
    assert "Vendor2" in result
    assert "cve?vendor=vendor2" in result


def test_vendors_excerpt_with_more():
    result = _excerpt(
        [
            "vendor1",
            "vendor2",
            "vendor3",
            "vendor4",
            f"vendor1{PRODUCT_SEPARATOR}product1",
            f"vendor2{PRODUCT_SEPARATOR}product2",
            f"vendor3{PRODUCT_SEPARATOR}product3",
            f"vendor4{PRODUCT_SEPARATOR}product4",
        ],
        "vendors",
    )

    assert '<span class="badge badge-primary">4</span>' in result
    assert "Vendor1" in result
    assert "cve?vendor=vendor1" in result
    assert "Vendor2" in result
    assert "cve?vendor=vendor2" in result
    assert "Vendor3" in result
    assert "cve?vendor=vendor3" in result
    assert "<i>and 1 more</i>" in result


def test_products_excerpt_without_more():
    result = _excerpt(
        [
            "vendor1",
            "vendor2",
            f"vendor1{PRODUCT_SEPARATOR}product1",
            f"vendor2{PRODUCT_SEPARATOR}product2",
        ],
        "products",
    )

    assert '<span class="badge badge-primary">2</span>' in result
    assert "Product1" in result
    assert "cve?vendor=vendor1&product=product1" in result
    assert "Product2" in result
    assert "cve?vendor=vendor2&product=product2" in result


def test_products_excerpt_with_more():
    result = _excerpt(
        [
            "vendor1",
            "vendor2",
            "vendor3",
            "vendor4",
            f"vendor1{PRODUCT_SEPARATOR}product1",
            f"vendor2{PRODUCT_SEPARATOR}product2",
            f"vendor3{PRODUCT_SEPARATOR}product3",
            f"vendor4{PRODUCT_SEPARATOR}product4",
        ],
        "products",
    )

    assert '<span class="badge badge-primary">4</span>' in result
    assert "Product1" in result
    assert "cve?vendor=vendor1&product=product1" in result
    assert "Product2" in result
    assert "cve?vendor=vendor2&product=product2" in result
    assert "Product3" in result
    assert "cve?vendor=vendor3&product=product3" in result
    assert "<i>and 1 more</i>" in result



from opencve.models import get_uuid
from opencve.models.changes import Change


def test_list_no_cves(client):
    response = client.get("/cve")
    assert response.status_code == 200
    assert b"No CVE found." in response.data


def test_list_cves(client, create_cves):
    cves = ["CVE-2018-18074", "CVE-2020-9392", "CVE-2020-26116"]
    vendors = ["canonical", "python-requests", "supsystic", "fedoraproject", "python"]
    products = [
        "ubuntu_linux",
        "requests",
        "pricing_table_by_supsystic",
        "fedora",
        "python",
    ]
    create_cves(cves)

    response = client.get("/cve")
    assert response.status_code == 200

    for word in cves + vendors + products:
        assert word in response.data.decode("utf-8")


def test_get_cve_not_found(client):
    response = client.get("/cve/404")
    assert response.status_code == 404
    assert b"Page not found" in response.data


def test_get_cve(client, create_cve, create_cwe):
    create_cwe(
        "CWE-522",
        "Insufficiently Protected Credentials",
        "The product transmits or stores authentication credentials, but it uses an insecure method that is susceptible to unauthorized interception and/or retrieval.",
    )
    create_cve("CVE-2018-18074")
    response = client.get("/cve/CVE-2018-18074")

    assert b"CVE-2018-18074" in response.data
    assert b"The Requests package before 2.20.0 for Python" in response.data
    assert b"5.0" in response.data
    assert b"9.8" in response.data
    assert b"http://docs.python-requests.org" in response.data
    assert b"cpe:2.3:a:python-requests:requests:*:*:*:*:*:*:*:*" in response.data
    assert b"CWE-522" in response.data
    assert b"Insufficiently Protected Credentials" in response.data
    assert b"<strong>canonical</strong>" in response.data
    assert b"<li>ubuntu_linux</li>" in response.data
    assert b"<strong>python-requests</strong>" in response.data
    assert b"<li>requests</li>" in response.data
    assert b"No history." in response.data


def test_get_change_not_found(client, create_cve, handle_events):
    response = client.get(f"/cve/CVE-2000-1234/changes/{get_uuid()}")
    assert response.status_code == 404

    create_cve("CVE-2018-18074")
    response = client.get("/cve/CVE-2018-18074/changes/not_found")
    assert response.status_code == 404

    response = client.get(f"/cve/CVE-2018-18074/changes/{get_uuid()}")
    assert response.status_code == 404

    handle_events("modified_cves/CVE-2018-18074_summary.json")
    change = Change.query.first()
    response = client.get(f"/cve/CVE-2018-18074/changes/{change.id}")
    assert response.status_code == 200


def test_get_change(client, create_cve, handle_events, make_soup):
    create_cve("CVE-2018-18074")
    handle_events("modified_cves/CVE-2018-18074_summary.json")
    change = Change.query.first()

    response = client.get(f"/cve/CVE-2018-18074/changes/{change.id}")
    assert response.status_code == 200
    assert b"CVE-2018-18074" in response.data
    assert b"List of Events" in response.data
    assert b"JSON Diff" in response.data

    soup = make_soup(response.data)
    assert sorted([s.text for s in soup.find("table").find("tr").find_all("td")]) == [
        "Summary",
        "The Requests package before 2.20.0 for Python sends an HTTP Authorization header to an http URI upon receiving a same-hostname https-to-http redirect, which makes it easier for remote attackers to discover credentials by sniffing the network.",
        "The summary has been changed.",
    ]



"""Make Cwe.cwe_id field not nullable

Revision ID: 41ad8e9163d4
Revises: 8bdc527d8d49
Create Date: 2021-01-16 18:08:17.844952

"""

# revision identifiers, used by Alembic.
revision = "41ad8e9163d4"
down_revision = "8bdc527d8d49"

from alembic import op
import sqlalchemy as sa


def upgrade():
    """Make Cwe.cwe_id field not nullable and add an index on it."""
    op.alter_column("cwes", "cwe_id", existing_type=sa.VARCHAR(), nullable=False)
    op.create_index(op.f("ix_cwes_cwe_id"), "cwes", ["cwe_id"], unique=False)


def downgrade():
    """Make Cwe.cwe_id field nullable and remove the index on it."""
    op.drop_index(op.f("ix_cwes_cwe_id"), table_name="cwes")
    op.alter_column("cwes", "cwe_id", existing_type=sa.VARCHAR(), nullable=True)



from opencve.extensions import db
from opencve.models.products import Product
from opencve.models.vendors import Vendor


def test_list_subscriptions_authentication(client, create_user):
    create_user("opencve")
    response = client.get("/api/account/subscriptions/vendors")
    assert response.status_code == 401
    response = client.get("/api/account/subscriptions/products")
    assert response.status_code == 401

    response = client.login("john").get("/api/account/subscriptions/vendors")
    assert response.status_code == 401
    response = client.login("john").get("/api/account/subscriptions/products")
    assert response.status_code == 401

    response = client.login("opencve").get("/api/account/subscriptions/vendors")
    assert response.status_code == 200
    response = client.login("opencve").get("/api/account/subscriptions/products")
    assert response.status_code == 200


def test_list_vendors_subscriptions(client, create_user, handle_events):
    user = create_user("opencve")
    response = client.login("opencve").get("/api/account/subscriptions/vendors")
    assert response.status_code == 200
    assert response.json == []

    handle_events("modified_cves/CVE-2018-18074.json")
    user.vendors.append(Vendor.query.filter_by(name="canonical").first())
    db.session.commit()

    response = client.login("opencve").get("/api/account/subscriptions/vendors")
    assert response.status_code == 200
    assert response.json == [{"human_name": "Canonical", "name": "canonical"}]


def test_list_products_subscriptions(client, create_user, handle_events):
    user = create_user("opencve")
    response = client.login("opencve").get("/api/account/subscriptions/products")
    assert response.status_code == 200
    assert response.json == []

    handle_events("modified_cves/CVE-2018-18074.json")
    user.products.append(Product.query.filter_by(name="requests").first())
    db.session.commit()

    response = client.login("opencve").get("/api/account/subscriptions/products")
    assert response.status_code == 200
    assert response.json == [
        {
            "name": "requests",
            "human_name": "Requests",
            "parent_vendor": "python-requests",
        }
    ]



import string

from flask import abort

from opencve.controllers.base import BaseController
from opencve.models.tags import UserTag


class UserTagController(BaseController):
    model = UserTag
    order = [UserTag.name.asc()]
    per_page_param = "TAGS_PER_PAGE"
    schema = {
        "user_id": {"type": str},
        "name": {"type": str},
    }

    @classmethod
    def build_query(cls, args):
        return cls.model.query.filter_by(user_id=args.get("user_id")), {}



import gzip
import json
from io import BytesIO

import arrow
import requests

from opencve.commands import header, info, timed_operation
from opencve.commands.imports.cpe import get_slug
from opencve.extensions import db
from opencve.utils import convert_cpes, flatten_vendors, get_cwes
from opencve.models import get_uuid
from opencve.models.changes import Change
from opencve.models.cve import Cve
from opencve.models.tasks import Task

NVD_CVE_URL = "https://nvd.nist.gov/feeds/json/cve/1.1/nvdcve-1.1-{year}.json.gz"


def run():
    """
    Import the CVE list.
    """
    mappings = {"vendors": {}, "products": {}}

    from opencve.commands.imports import CURRENT_YEAR, CVE_FIRST_YEAR

    # Create the initial task
    task = Task()
    db.session.add(task)
    db.session.commit()
    task_id = task.id

    for year in range(CVE_FIRST_YEAR, CURRENT_YEAR + 1):
        header("Importing CVE for {}".format(year))
        mappings.update({"cves": [], "changes": []})

        # Download the file
        url = NVD_CVE_URL.format(year=year)
        with timed_operation("Downloading {}...".format(url)):
            resp = requests.get(url).content

        # Parse the XML elements
        with timed_operation("Parsing JSON elements..."):
            raw = gzip.GzipFile(fileobj=BytesIO(resp)).read()
            del resp
            items = json.loads(raw.decode("utf-8"))["CVE_Items"]
            del raw

        with timed_operation("Creating model objects..."):

            for item in items:
                cve_db_id = get_uuid()
                summary = item["cve"]["description"]["description_data"][0]["value"]
                cvss2 = (
                    item["impact"]["baseMetricV2"]["cvssV2"]["baseScore"]
                    if "baseMetricV2" in item["impact"]
                    else None
                )
                cvss3 = (
                    item["impact"]["baseMetricV3"]["cvssV3"]["baseScore"]
                    if "baseMetricV3" in item["impact"]
                    else None
                )

                # Construct CWE and CPE lists
                cwes = get_cwes(
                    item["cve"]["problemtype"]["problemtype_data"][0]["description"]
                )
                cpes = convert_cpes(item["configurations"])
                vendors = flatten_vendors(cpes)

                # Create the CVEs mappings
                mappings["cves"].append(
                    dict(
                        id=cve_db_id,
                        cve_id=item["cve"]["CVE_data_meta"]["ID"],
                        summary=summary,
                        json=item,
                        vendors=vendors,
                        cwes=cwes,
                        cvss2=cvss2,
                        cvss3=cvss3,
                        created_at=arrow.get(item["publishedDate"]).datetime,
                        updated_at=arrow.get(item["lastModifiedDate"]).datetime,
                    )
                )

                # Create the vendors and their products
                for vendor, products in cpes.items():

                    # Create the vendor
                    if vendor not in mappings["vendors"].keys():
                        mappings["vendors"][vendor] = dict(id=get_uuid(), name=vendor)

                    for product in products:
                        if get_slug(vendor, product) not in mappings["products"].keys():
                            mappings["products"][get_slug(vendor, product)] = dict(
                                id=get_uuid(),
                                name=product,
                                vendor_id=mappings["vendors"][vendor]["id"],
                            )

        # Insert the objects in database
        with timed_operation("Inserting CVE..."):
            db.session.bulk_insert_mappings(Cve, mappings["cves"])
            db.session.commit()

            # Create the changes based on CVEs data
            for cve in mappings["cves"]:
                mappings["changes"].append(
                    dict(
                        id=get_uuid(),
                        created_at=cve["created_at"],
                        updated_at=cve["updated_at"],
                        json=cve["json"],
                        cve_id=cve["id"],
                        task_id=task_id,
                    )
                )
            db.session.bulk_insert_mappings(Change, mappings["changes"])
            db.session.commit()

        info("{} CVE imported.".format(len(mappings["cves"])))

        # Free the memory after each processed year
        del mappings["cves"]
        del mappings["changes"]

    return mappings
